
/*****************************************************************************/
/* COMBOX.X:  Macro to draw a comment box.  Invoke with ? for brief help.    */
/*                                                                           */
/* Author: Craig Raymond (CRAYMOND at GDLVM6)                                */
/*****************************************************************************/
/* Syntax: COMBOX number_of_lines border_character                           */
/*                                                                           */
/* COMBOX will create a comment box from the current cursor position to the  */
/*        right comment margin (as determined from X.PRO). The qty of blank  */
/*        lines in the comment box defaults to 2, the 'number_of_lines'      */
/*        input overrides this value.  The border character can also be      */
/*        redefined with the 'border_character' input (this must be one      */
/*        character only). A blank line will be inserted above and below the */
/*        comment box.  This can be assigned to a PFKey for convenience.     */
/*****************************************************************************/
/* Revision history:                                                         */
/* January   04, 1994 - Original release.                              (1.00)*/
/* January   07, 1994 - With Mel Rothman's help, removed extraneous    (1.01)*/
/*                      lines of code, simplified string creation, and       */
/*                      improved performance (at least I think the           */
/*                      performance has improved.  Many thanks Mel!          */
/* January   10, 1994 - Use new 'EXTRACT/MARGINS & COMMENTS/' ability  (1.02)*/
/*                      of X to pull in the margins and comment chars        */
/*                      defined for the file in question. Added ability      */
/*                      to input an alternate border character.              */
/* January   14, 1994 - Correct length of comment string (was short by (1.03)*/
/*                      by one char.                                         */
/*****************************************************************************/

parse arg user_lines user_char

/*****************************************************************************/
/* Default parameters.                                                       */
/*****************************************************************************/

border_char = "*"                                /* Top & bot line character */
empty_lines = 2                                  /* Number of empty lines    */

/*****************************************************************************/
/* Get ending column.                                                        */
/*****************************************************************************/

'EXTRACT /MARGINS/'
end_col = MARGINS.4                              /* Last column for comment  */

/*****************************************************************************/
/* Check input arguments, and change parameters as required.                 */
/*****************************************************************************/

If user_lines == '?'                             /* If lines is a '?', then  */
   Then signal DispHelp                          /* display brief help       */

If (datatype(user_lines,'W'))                    /* If lines is a whole #,   */
   Then empty_lines = user_lines                 /* set to users # of lines  */

If user_char <> '' Then                          /* If bord_char is defined  */
   If length(user_char) == 1                     /* and is only 1 character  */
      Then border_char = user_char               /* use it instead of deflt  */
      Else Do
         MSG 'Border character can only be one character.'
         Exit 99
      End

/*****************************************************************************/
/* Get comment characters, and remove any blanks from comment char strings.  */
/*****************************************************************************/

'EXTRACT /COMMENTS/'
coml = COMMENTS.1                                /* Left comment chars       */
comr = COMMENTS.2                                /* Right comment chars      */
coml = strip(coml,b,)                            /* Remove blanks            */
comr = strip(comr,b,)                            /* Remove blanks            */
lcml = length(coml)                              /* length o'left cmnt strng */
rcml = length(comr)                              /* length o'rght cmnt strng */

/*****************************************************************************/
/* Get cursor position (needed for # of blanks), compute where beginning of  */
/* comment box is, compute length of comment chars/blanks string.            */
/*****************************************************************************/

COMMAND 'EXTRACT /CURSOR/'
blank_cols = CURSOR.2                            /* Number of blanks on left */
qty_coms = end_col - lcml - rcml - blank_cols +1 /* Number of middle chars   */

/*****************************************************************************/
/* Make leading blanks string, top & bottom comment box strings, and middle  */
/* (or empty) comment box strings.                                           */
/*****************************************************************************/

blankstring = copies(' ',blank_cols)
topbottomline = blankstring||coml||copies(border_char,qty_coms)||comr
emptyline = blankstring||coml||copies(' ',qty_coms)||comr

/*****************************************************************************/
/* Draw blank line and top line.                                             */
/*****************************************************************************/

'INPUT'
'INPUT'topbottomline

/*****************************************************************************/
/* Draw quantity of desired empty lines.                                     */
/*****************************************************************************/

Do empty_lines
   'INPUT'emptyline
end

/*****************************************************************************/
/* Draw bottom line and blank line.                                          */
/*****************************************************************************/

'INPUT'topbottomline
'INPUT'

/*****************************************************************************/
/* Move cursor to first 'editing' column and row of comment box.             */
/*****************************************************************************/

empty_lines = empty_lines + 1                    /* Lines to move up         */
blank_cols = blank_cols + 3                      /* Cols to move right       */

'CURSOR -'empty_lines blank_cols

exit 0

/*****************************************************************************/
/* Display brief help message.                                               */
/*****************************************************************************/

DispHelp:
   MSG 'Syntax is: COMBOX Number_of_lines .... View COMBOX.X for details.'
exit 0
