/*--------------------------------------------------------------------*/
/* LOAD.X - by Erich Fussi (FUSSI at SDFVM1)                          */
/*                                                                    */
/*   Enhances X's edit command.                                       */
/*   Handles directory listings, #include statements, compiler        */
/*   messages, and files related to the current one (see below).      */
/*   Retains the special meaning of the '=' character in file         */
/*   specifications. Easily extendible.                               */
/*                                                                    */
/*   If a file is not found in the current directory, the appropriate */
/*   path from the environment is searched.                           */
/*                                                                    */
/*   To install it, add the following to your X profile:              */
/*     synonym edit = macro load                                      */
/*   Finally, assign a key to it                                      */
/*     key f5 = macro load                                            */
/*                                                                    */
/*   If you are using the hostedit macro use the following:           */
/*     synonym edit = macro hostedit macro load                       */
/*     synonym save = macro hostedit command save                     */
/*   Additionally, you have to make the following changes in          */
/*   hostedit.x (affected lines marked with @X):                      */
/*                                                                    */
/*     Parse Upper Arg cmd op fn   @X insert 'cmd'                    */
/*                                                                    */
/*     Parse Var fn drive ':' file                                    */
/*     If Substr(drive,1,1) <> 'H' | file = '' | Pos('\',fn) > 0      */
/*       Then Do                                                      */
/*         cmd op fn               @X insert 'cmd'                    */
/*         Exit                                                       */
/*       End                                                          */
/*     :                                                              */
/*     :                                                              */
/*     If op = 'LOAD' Then Do      @X change from 'EDIT' to 'LOAD'    */
/*                                                                    */
/*                                                                    */
/*   Assigning this macro to a key is especially useful for compiler  */
/*   messages and 'related files'. For example, if you are editing    */
/*   foo.cxx, pressing F5 will search the INCLUDE path first for      */
/*   foo.hxx then for foo.h. If in turn you are editing foo.hxx,      */
/*   pressing F5 will search the DPATH path for foo.cxx.              */
/*   If you invoke X with 'foo.x' as parameter and foo.x does not     */
/*   exist in the current directory, the PATH will be searched for    */
/*   it. If foo.x already exists in your PATH, and you want to create */
/*   a new foo.x locally, type 'X .\foo.x'.                           */
/*                                                                    */
/*   Note: This macro works best if you also have a 'DIR' macro that  */
/*         captures the output from a 'dir' command in a file.        */
/*                                                                    */
/*   History:                                                         */
/*     1995/11/25 EF initial version ported from my load.e            */
/*     1995/11/25 EF fixed typos/omissions in header                  */
/*     1995/11/28 EF corrected installation instructions, improved    */
/*                   icc and bcc message parsing                      */
/*     2001/03/22 ESR Fixed loading long file names with spaces       */
/*--------------------------------------------------------------------*/

/* path to search for various file types (extend this as needed) -----*/

path.       = 'DPATH'
path.H      = 'INCLUDE'
path.HXX    = 'INCLUDE'
path.HPP    = 'INCLUDE'
path.TXX    = 'INCLUDE'
path.TPP    = 'INCLUDE'
path.X      = 'PATH'
path.CMD    = 'PATH'

/* related file types (extend this as needed) ------------------------*/

related.    = ''
related.C   = 'H HXX'
related.CXX = 'HXX H TXX'
related.CPP = 'HPP H TPP'
related.H   = 'C CXX CPP RC DLG'
related.HXX = 'CXX TXX'
related.HPP = 'CPP TPP'
related.TXX = 'HXX'
related.TPP = 'HPP'
related.RC  = 'H'
related.DLG = 'H'

/* initialization ----------------------------------------------------*/

parse arg file
from = 'argument'

cmd = 'edit'                                                    /* or 'dir'*/
fromdir = 0                        /* is the file from a directory listing?*/
pos = ''                                     /* cursor position in the file*/
error = ''                                         /* error text to display*/

'extract /curline/'

parse var curline.1 line

/* try #include statement --------------------------------------------*/

if file = '' then
do
  parse var line '#include' file .

  if file <> '' then
  do
    file = substr(file, 2, length(file) - 2)
    from = '#include stmt'
  end
end

/* try icc error message ---------------------------------------------*/

if file = '' then
do
  parse var line file '(' pos ')' . . . error
  pos = translate(pos, ' ', ':')

  if (error = '') | \datatype(word(pos, 1), 'w') then
  do
    file = ''
    pos = ''
    error = ''
  end

  from = 'icc error'
end

/* try bcc error message ---------------------------------------------*/

if file = '' then
do
  parse var line 'Warning ' file ' ' pos ':' error

  if error = '' then
    parse var line 'Error ' file ' ' pos ':' error

  if (error = '') | \datatype(pos, 'w') then
  do
    file = ''
    pos = ''
    error = ''
  end

  from = 'bcc error'
end

/* try HPFS directory listing ----------------------------------------*/

if file = '' then
do
  parse var line 3 ds1 +1 6 ds2 +1 13 ts +1 . sz .

  if (ds1 = '-') & (ds1 = ds2) & (ts = ':') then
    parse var line 41 file

  if file <> '' then
    fromdir = 1

  if sz = '<DIR>' then
    cmd = 'dir'

  from = 'HPFS dir'
end

/* try FAT directory listing -----------------------------------------*/

if file = '' then
do
  parse var line 13 sz . 26 ds1 +1 29 ds2 +1 36 ts +1

  if (ds1 = '-') & (ds1 = ds2) & (ts = ':') then
  do
    parse var line file ext . 13 .

    if ext <> '' then
      file = file'.'ext
  end

  if file <> '' then
    fromdir = 1

  if sz = '<DIR>' then
    cmd = 'dir'

  from = 'FAT dir'
end

/* try related files -------------------------------------------------*/

if file = '' then
do
  'extract /name/'

  name = filespec('name', name.1)
  parse upper var name name '.' ext

  exts = related.ext

  do while (file = '') & (exts <> '')
    parse var exts ext exts
    file = lookup(name'.'ext)
  end

  from = 'related'
end

/* try word under the cursor -----------------------------------------*/

if file = '' then
do
  'extract /cursor/'
  pos.1 = cursor.1
  pos.2 = cursor.2

  'cursor' pos.1 pos.2 + 1
  'previous_word'
  'extract /cursor/'

  parse var line =(cursor.2) file .

  'cursor' pos.1 pos.2

  from = 'current line'
end

/* try the openfile command if we found nothing ----------------------*/

if file = '' then
do
  'command openfile'
  exit rc
end

/* add the appropriate path to the file(s) ---------------------------*/

if fromdir = 1 then
do
  'extract /cursor/'
  'top'
  'l /:\/'

  if rc = 0 then
  do
    'extract /curline/'
    parse var curline.1 . . dir
    file = mkpath(dir, file)
  end

  'restore_find'
  'cursor' cursor.1 cursor.2
end
else
do
  files = file
  file = ''

  do while (files <> '')
    parse var files f1 files
    fn = lookup(f1)

    /* Start ESR */
    /* Test for blanks in path name */
    If pos(' ', fn) > 0
      Then fn = '"'||fn||'"'
    /* End ESR */

    if fn = '' then
      file = file f1
    else
      file = file fn
  end
end

/* load the file -----------------------------------------------------*/

cmd file
cmd_rc = rc

if pos <> '' then
  'cursor' pos

if error <> '' then
  'msg' error

/*msg cmd "'"file"' ("from")" */

exit cmd_rc

/* mkpath procedure --------------------------------------------------*/

mkpath: procedure

  parse arg dir, file

  if (substr(dir, length(dir)) <> '\') then
    dir = dir'\'

  if (file <> '.') & (file <> '..') then
    return translate(dir||file)

  skip = length(file)

  parse var dir drv '\' dir
  parse value reverse(dir) with =(skip) '\' dir

  dir = drv'\'reverse(dir)

  return dir

/* lookup procedure --------------------------------------------------*/

lookup: procedure expose path.

  parse arg name
  parse upper var name . '.' ext

  /* Start ESR */
  parse source system .
  select
    when system = "OS/2"
      then env = "os2environment"
    when left(translate(system), 6) = "WINDOW"
      then env = "environment"
    otherwise env = "unknown"
  end
  /* End ESR */

  file = stream(name, 'c', 'query exists')
  path = '.;'value(path.ext,, env)

  do while (file = '') & (path <> '')
    parse var path dir ';' path

    if dir = '' then
      iterate

    file = mkpath(dir, name)
    file = stream(file, 'c', 'query exists')
  end

  return file
