/******************************************************************/
/* Math.X                             Toby Thurston - 19 Nov 1997 */
/******************************************************************/
/* Math does sums for you & inserts the results at the cursor.    */
/* Use any REXX operators + - * / % **, but use \\ for //         */
/*   because of a stupid OS/2 limitation on the REXX cmd line     */
/* Updated by Morten Kjrulff (mkj@maerskdata.dk) for Regina Rexx */
/******************************************************************/
arg expr                               /* read the expression     */
if expr = ?
  then do
    parse source os calltype program
    'edit' program "/c16"
    'pagedown'
    exit
  end
/******************************************************************/
/* Typical usage:  move cursor to where you want the results, ESC */
/* to bring up the command line, then:                            */
/*                                                                */
/*  math 2+3      inputs 5                                        */
/*  math 2/3.0    inputs 0.7    note the decimal points           */
/*  math 2.03 / 3 inputs 0.68      and the commas                 */
/*  math 2,000*34 inputs 68,000                                   */
/*  math 2+#      inputs 2 + the first number in a marked area    */
/*                                                                */
/* If you use a # in the expression then Math.x looks for a mark, */
/* finds the numbers in it and substitutes them for the #s in the */
/* input expression.  It gives an error if no mark or not enough  */
/* numbers.  If the number of numbers in a line matches the       */
/* number of #s AND the mark extends for more than one row down,  */
/* then the operation will be repeated all down the mark.  This   */
/* is good for doings ops on columns of numbers.                  */
/*                                                                */
/* You can also use REXX BIFs or any other program that you have  */
/* written to be called as a routine.  So for example you have    */
/* two cols of data like this:                                    */
/*                                                                */
/*          234   356   and you want the max of each    356       */
/*          123    34   col.  Mark the data move the    123       */
/*          297   403   cursor to where you want the    403       */
/*          123   405   output, ESC to cmd line & do    405       */
/*          117   107   "math max(#,#)" to get ---->    117       */
/*                                                                */
/* Providing you are NOT using a BIF or a function then, K M and  */
/* G can be used for *(2**10), *(2**20) & *(2**30).  For example: */
/* example "math 2K" --> 2048, "math 150 * 300000/(1K)" --> 43945 */
/*                                                                */
/* Math provides a couple of extra functions for you as internal  */
/* routines, currently I've implemented only SQRT.  So you can    */
/* use "sqrt(2)" etc in your sums (but be sure to use enough DPs) */
/******************************************************************/
expr = change(expr,"\\","//")

if pos("#",expr) = 0 then do           /* normal mode             */
  'cursor data'
  'keyin' workout(expr)
  exit
end
                                       /* substitution mode       */
'extract /mark/'                       /* look for a mark         */
select                                 /* check mark is ok        */
  when mark.0 = 0 then "msg Nothing marked! I need a mark to substitute the #s"
  when mark.6 = 0 then "msg Mark exists off screen in" mark.1
otherwise                              /* mark is ok now          */
  'cursor data'
  'extract /cursor/'                   /* save the cursor         */
  do i = mark.2 to mark.3
    'extract /line' i'/'               /* read the line           */
    if mark.4 > 0                      /* if its a block mark     */
      then data = substr(line.1,mark.4,mark.5-mark.4+1)
      else data = line.1
    data = onlynums(data)              /* remove alpha words etc  */
    if data = "" then iterate i
    thisexpr = expr
    do j = 1 until pos("#",thisexpr) = 0       /* replace the #s  */
      qmark = word(data,j)
      if qmark = ""                    /* run out of numbers in data? */
        then do
          'msg Not enough numbers in marked data on line' i
          iterate i
        end
      parse var thisexpr prefix "#" suffix
      thisexpr = prefix""qmark""suffix
    end
    'msg Processing line' 1+i-mark.2 'of' 1+mark.3-mark.2
    'cursor' choose(mark.2=mark.3,cursor.1,i) cursor.2
    'keyin' workout(thisexpr)
  end
  'msg'
  'cursor' cursor.1 cursor.2
end
exit

syntax:
'msg' expr 'caused a REXX syntax error' condition("d")
exit

contains_a_fn: procedure  /* does this expr have a fn call in it? */
  arg expr
  qmark = pos("(",expr)
  do while qmark>1          /* no ( or first char ( means no function */
    if datatype(substr(expr,qmark-1,1),"U")/* letter before (         */
      then return 1                    /* got a function          */
    qmark = pos("(",expr,qmark+1)
  end
  return 0

workout: procedure
  arg expr
  fnqmark = contains_a_fn(expr)
  commaqmark = \fnqmark & pos(",",expr) > 0    /* got any commas ?        */
  if \fnqmark
    then do
      expr = change(expr,"G","*1073741824")    /* G for Gigabytes */
      expr = change(expr,"M","*1048576")       /* M for Megabytes */
      expr = change(expr,"K","*1024")  /* K for Kilobytes         */
    end
  if commaqmark then expr = change(expr,",")
  numeric digits 15
  signal on syntax
  interpret 'x=format('expr',,'dp(expr)')'
  signal off syntax
  if commaqmark then x = addcommas(x)
return x

addcommas: procedure
  parse arg int "." decimal
  out = ""
  do while length(int) > 3
    out = ","right(int,3)""out
    int = left(int,length(int)-3)
  end
  out = int""out
  if decimal <> "" then out = out"."decimal
return out

Change: procedure
  parse arg string, old, new
  out=''
  do while pos(old,string) <> 0
    parse var string prefix (old) string
    out=out""prefix""new
  end
return out""string
                   /* This trick comes from "Programming in REXX" */
Onlynums: procedure      /* Charles Daney, McGraw Hill 1992, p129 */
return space(translate(arg(1),,"/"delstr(xrange(),c2d(".")+1,12)))

dp: procedure   /* finds the most decimal places in an expression */
 arg expr
 dp = 0
 expr = onlynums(expr)
 do i = 1 to words(expr)
   parse value word(expr,i) with . "." qmark
   dp = max(dp,length(qmark))
 end
 arg expr
 if pos("/",expr) > 0 then dp = dp + 1
 if pos("SQRT",expr) then dp = dp + 2
return dp

choose: return arg(arg()-arg(1))

sqrt: procedure expose expr            /* square root             */
  arg x
  if x = 0 then return 0
  if x < 0
    then do; call beep 100,200 ; 'msg Can''t do root of -ve number:' expr ; exit; end
  sqrt = x/10
  old  = 0
  do until sqrt = old
    Old = sqrt
    Sqrt=(Sqrt+X/Sqrt)/2
  end
return sqrt
