/********************************* PAINTCOM ********************************/
/*                        X2 Editor Macro PAINTCOM                         */
/*                     PAINT the COMments in a program                     */
/*                                                                         */
/* PAINTCOM:  An editor macro to Paint the comments in some programming    */
/* languages where the editor's normal comment recognition mechanism       */
/* (using comment_prefix and comment_suffix character strings) has         */
/* difficulty.  The normal mechanism does not, for example, recognise      */
/* multi-line or nested comments as in REXX or comments that do not have   */
/* prefix or suffix character strings, for example in JCL or BAL (host     */
/* Assembler) where any character string after the normal statements is    */
/* regarded as a comment.  For a list of supported languages, see below.   */
/*                                                                         */
/* If a mark (line mark only, no block marks) is present, only the marked  */
/* lines are processed, otherwise the whole file.                          */
/*                                                                         */
/* Call                                                                    */
/*                                                                         */
/*     paintcom < ?  Paint  Assembler  C  Jcl  Rexx  Undo >          */
/*                                                                         */
/* whereby all arguments are mutually exclusive and their meanings are:    */
/*                                                                         */
/* ?         tells this macro to put out these comments as a help text.    */
/*                                                                         */
/* Paint     instructs this macro to try to determine the file type and    */
/*           colour the comments appropriately.  This is the default.      */
/*           This macro first checks the code_type setting for the file    */
/*           being edited, and if that is 'NONE', this macro will then     */
/*           check the file's extension.  JCL and (Host) Assembler are     */
/*           recognised by means of their extensions and so you may wish   */
/*           to customise the list of extensions, see 'customization       */
/*           section' below.                                               */
/*                                                                         */
/* Assembler tells this macro to assume the text contains host (Mainframe) */
/*           Assembler statements and to colour the text accordingly.      */
/*                                                                         */
/* C         tells this macro to assume C style comments (multi-line       */
/*           constructs are allowed but no nesting).                       */
/*                                                                         */
/* Jcl       tells this macro to assume the text contains MVS JCL and to   */
/*           colour the text accordingly.                                  */
/*                                                                         */
/* Rexx      tells this macro to assume REXX style comments (multi-line    */
/*           and nested constructs are allowed)                            */
/*                                                                         */
/* Undo      instructs this macro to remove its results from the Paint     */
/*           operation.  Note that this will remove all colouring that     */
/*           resulted from LINECOLOUR, not just from this macro.           */
/*                                                                         */
/* Any abbreviation of the available options down to just the first        */
/* character only are supported.                                           */
/*                                                                         */
/* Change Activity                                                         */
/* 00-02-22 Roger Spencer, New                                             */
/* 00-02-29 Roger Spencer, Added JCL, Assembler and C style painting.      */
/***************************************************************************/

Signal Endhelpline                     /* end of help text                 */
ENDHELPLINE: lasthelpline = sigl -1

/*-------------------------------------------------------------------------*/
/* N O T E !  keep the above two lines as they are used to pick up         */
/* the number of the last line of help text to put out to a window.        */
/*-------------------------------------------------------------------------*/

/* this was the first macro to be written with the help of SKELETON.X      */

Signal On Novalue; Signal On Syntax

/* ************************ customization section ************************ */
/*         define the extensions for recognising JCL and Assembler         */
/*                                                                         */
assembler_ft = 'ASS A37'               /* Assembler file types             */
jcl_ft       = 'MVS JCL'               /* JCL file types (extensions)      */

false = 0; true = 1                    /* Boolean values                   */

"EXTRACT /FILEINFO/"
If (rc <> 0) | (fileinfo.1 = '') Then Do
  fileinfo.1 = '????'                  /* cannot determine name            */
End                                    /* end if                           */

"EXTRACT /ESCAPE/"

/*********************** Get the name of this macro ************************/

/*  Define in the variable 'filespec_os' the names of systems as returned  */
/* by 'EXTRACT /OS/' that support the filespec() function.  Note that the  */
/*  use of the filespec() function is a known system dependency in this    */
/*  macro, however, it is protected by checking the system we are running  */
/*                                 under.                                  */

filespec_os = 'OS/2 WINDOWS NT WINDOWS 95'

'EXTRACT /OS/'
If (wordpos(translate(os.1), filespec_os) = 0) Then Do
  myname = fileinfo.1                  /* filespec not supported           */
End                                    /* end if                           */
Else Do
  myname = Translate(Filespec('Name', fileinfo.1))
  Parse Var myname myname '.' .        /* drop extension                   */
End                                    /* end else                         */

/************************** check if help wanted ***************************/

Arg parm                /* assume opts to be passed to editor at invocation*/

'EXTRACT /FLSCREEN/'                   /* get no. of lines on screen       */
If (flscreen.0 <> 2)
  Then window_size = 18                /* set up no. of lines...           */
  Else Do                              /* ...for pop-up windows            */
    window_size = (flscreen.2 - flscreen.1 + 1)- 4
  End

help = '? /? -? HELP'                  /* Help text wanted ?      .        */
If (wordpos(parm, help) <>0) Then Do
  window_width = Length(Sourceline(1))
  'WINDOW' window_size window_width lasthelpline myname 'Help text'
  Do i = 1 To lasthelpline-1           /* make sure loop ends sometime     */
    'WINLINE' Sourceline(i) || '\nREFRESH'
  End
  Exit 0
End

/************* set the variable 'op' to indicate the optione ***************/

op = 'PAINT'                           /* Default operation                */

Select

  When (abbrev('PAINT',parm)) Then Do  /* must be first in SELECT for..    */
    op = 'PAINT'                       /* ..defaultig                      */
  End                                  /* end if                           */

  When (abbrev('JCL',parm)) Then Do
    op = 'JCL'
  End                                  /* end if                           */

  When (abbrev('ASSEMBLER',parm)) Then Do
    op = 'ASSEMBLER'
  End                                  /* end if                           */

  When (abbrev('C',parm)) Then Do
    op = 'C'
  End                                  /* end if                           */

  When (abbrev('REXX',parm)) Then Do
    op = 'REXX'
  End                                  /* end if                           */

  When (abbrev('UNDO' ,parm)) Then Do
    op = 'UNDO'
  End                                  /* end if                           */

  Otherwise Do
    'MESSAGEBOX' myname' Illegal option:' parm', aborting'
    Exit 8
  End

End                                    /* end if                           */

/*********************** Get the range of lines ****************************/

'EXTRACT /FILEINFO/'

'EXTRACT /MARK/'
If (mark.0 = 0) Then Do                /* no marked area                   */
  first_line = 1                       /* start at top of file             */
  last_line  = fileinfo.2
End                                    /* end if                           */
Else Do
  If (mark.4 = 0) Then Do              /* it's a line mark                 */
    first_line = mark.2
    last_line  = mark.3
  End                                  /* end if                           */
  Else Do                        /* it's a block mark, we don't suport that*/
    'MESSAGEBOX' myname' Not a line mark, aborting'
    Exit 0
  End                                  /* end else                         */
End                                    /* end else                         */

'EXTRACT /CURSOR/'

/****************************** See to Reset *******************************/

If (op = 'UNDO') Then Do
  Do linei = first_line To last_line
    ''linei
    'LINECOLOUR'                       /* No operands resets line colour   */
  End                                  /* end do                           */
  'TOPLINE' flscreen.1
  'CURSOR' cursor.1 cursor.2
  Exit 0
End                                    /* end if                           */

/******************* pickup the current colours setting ********************/

'EXTRACT /COLOURS/'
If (rc <> 0) Then Do
  'MESSAGEBOX' myname' cannot pick up colours, aborting'
  Exit 8
End                                    /* end if                           */

cols. = '?'

Do coli = 1 To colours.0

  Parse Var colours.coli area setting
  cols.area = setting
  If (area = 'QUOTES') Then Do
    quotes_area_index = coli           /* remember for paint_assembler     */
  End                                  /* end if                           */

End                                    /* end do                           */

/* this next variable is for shifting in after a comment                   */
non_comm_attr = 'DATA'                 /* assume in edit mode              */
If (fileinfo.4 = 'BROWSE') Then Do
  non_comm_attr = 'BROWSE_DATA'        /* assume in edit mode              */
End                                    /* end if                           */

/************* If argument = 'PAINT' try to pick up file type **************/

If (op = 'PAINT') Then Do

  'EXTRACT /CODE_TYPE/'
  If (rc <> 0) Then Do
    'MESSAGEBOX' myname' Return code' rc 'from Extract, Aborting'
    Exit 8
  End                                  /* end if                           */

  If (code_type.1 = 'NONE') Then Do    /* Undefined                        */
    'EXTRACT /FT/'                     /* try file type                    */
    If (rc <> 0) Then Do
      'MESSAGEBOX' myname' Return code' rc 'from Extract, Aborting'
      Exit 8
    End                                /* end if                           */

    If (wordpos(ft.1,assembler_ft) <> 0) Then Do
      op = 'ASSEMBLER'
    End                                /* end if                           */

    If (wordpos(ft.1,jcl_ft) <> 0) Then Do
      op = 'JCL'
    End                                /* end if                           */

  End                                  /* end if                           */
  Else If (wordpos(code_type.1,'REXX C NETREXX') <> 0) Then Do
    op = code_type.1
  End                                  /* end else                         */

End                                    /* end if                           */

If (op = 'PAINT') Then Do              /* hasn't changed, so we cannot..   */
  'MESSAGEBOX' myname' Cannot identify language, aborting'
  Exit 8                               /* ..identify language              */
End                                    /* end if                           */

/******************* see to language specific processing *******************/
/*   When here, we have already seen to the options Undo (reset coloured   */
/*  lines) and Paint (we have set up the langugage in the variable "op").  */
/*                  Now call an appropriate subprocedure.                  */
Select

  When op = 'JCL' Then Do
    Call Paint_jcl
  End                                  /* end if                           */

  When op = 'ASSEMBLER' Then Do
    Call Paint_assembler
  End                                  /* end if                           */

  When op = 'REXX' Then Do
    Call Paint_rexx
  End                                  /* end if                           */

  When (op = 'C') | ( op = 'NETREXX') Then Do
    Call Paint_rexx
  End                                  /* end if                           */

End                                    /* end if                           */

/***************************************************************************/
/*          The following is the only exit in the non-error case           */

Exit 0

/************************** See to Painting REXX ***************************/
/*           Subprogram to paint comments in a REXX comment style          */
/*     This subprogram is also used for C, the only difference is that     */
/*        the check for nested comments is only performed for REXX         */
/***************************************************************************/

PAINT_REXX:
state = 'OPEN'                         /* in open code                     */

nest  = 0
scol = 0                               /* start colouring column           */
ecol = 0                               /* end colouring column             */
cs = '/' || '*'                        /* comment start chars              */
ce = '*' || '/'                        /* comment end chars                */

sq = "'"                               /* single quote                     */
dq = '"'                               /* double quote                     */

/* start loop through lines to paint                                       */
/* =================================                                       */

/* Painting is done by means of a finite state machine. The states, their  */
/* meanings are:                                                           */
/*                                                                         */
/* State  Meaning                     Action                               */
/*                                                                         */
/* 'SQ'   single quote detected       Scan for another single quote..      */
/*                                    (find end of quoted string)          */
/*                                                                         */
/* 'DQ'   double quote detected       Scan for another double quote        */
/*                                    (find end of quoted string)          */
/*                                                                         */
/* 'COMMENT' start of comment..       Scan for end of comment string or... */
/*           ..string detected        ..another start of comment string.   */
/*                                    Increment or decrement nesting-...   */
/*                                    ..level counter accordingly          */
/*                                                                         */
/* 'OPEN'   in open code             Scan for any of above, end of...      */
/*                                    ...comment is an error               */

Do linei = first_line To last_line

  ''linei
  'EXTRACT /CURLINE'                   /* get contents of current line     */

  If (length(curline.1) = 0) Then Do
    Iterate linei
  End                                  /* end if                           */

  If (state = 'COMMENT') Then Do       /* multi-line comment ..            */
    scol = 1                           /* ..continued on this line         */
  End                                  /* end if                           */

  Do coli = 1 To Length(curline.1)

    If (state = 'OPEN') Then Do        /* Just check for start of comment  */
      If (substr(curline.1,coli,2) = cs) Then Do
        state = 'COMMENT'
        nest = nest + 1
        scol = coli
        coli = coli + 1                /* scan past comment start          */
      End                              /* end if                           */
      If (substr(curline.1,coli,2) = ce) Then Do
        'MESSAGEBOX' myname' too many end of comment chars in line/col' ,
          linei'/'coli
        nest = nest - 1
        coli = coli + 1                /* scan past comment end chars      */
      End                              /* end if                           */
      If (substr(curline.1,coli,1) = sq) Then Do
        state = 'SQ'
      End                              /* end if                           */
      If (substr(curline.1,coli,1) = dq) Then Do
        state = 'DQ'
      End                              /* end if                           */
      Iterate coli
    End                                /* end if                           */

    If (state = 'COMMENT') Then Do     /* check for nesting only for REXX  */
      If (substr(curline.1,coli,2) = cs) ,
        &  (op = 'REXX') Then Do
        nest = nest + 1                /* nested comment                   */
        coli = coli + 1                /* scan past comment start          */
      End                              /* end if                           */
      If (substr(curline.1,coli,2) = ce) Then Do
        nest = nest - 1
        If (nest = 0) Then Do          /* into open code again             */
          Call Colour state scol coli+1
          state = 'OPEN'
          scol = 0
        End                            /* end if                           */
        coli = coli + 1                /* scan past comment end chars      */
      End                              /* end if                           */
      Iterate coli
    End                                /* end of state = 'comment'         */

    If (state = 'SQ') Then Do          /* single quote already detected    */
      If (substr(curline.1,coli,1) = sq) Then Do
        state = 'OPEN'
      End                              /* end if                           */
      Iterate coli
    End                                /* end if                           */

    If (state = 'DQ') Then Do          /* double quote already detected    */
      If (substr(curline.1,coli,1) = dq) Then Do
        state = 'OPEN'
      End                              /* end if                           */
      Iterate coli
    End                                /* end if                           */

  End                                  /* end of loop through one line     */

  If (state = 'COMMENT') Then Do       /* multi-line comment               */
    Call Colour state scol Length(curline.1)
  End                                  /* end if                           */

End                                    /* end of loop throug lines         */

If (nest < 0) Then Do
  'MESSAGEBOX' myname' too many end comment chars'
End                                    /* end if                           */
If (nest > 0) Then Do
  'MESSAGEBOX' myname' too many start of comment chars'
End                                    /* end if                           */

'TOPLINE' flscreen.1
'CURSOR' cursor.1 cursor.2

Return

/************************ See to Painting Assembler ************************/
/*          Subprogram to paint comments in an Assembler program           */
/***************************************************************************/

PAINT_ASSEMBLER:

/* start loop through lines to paint                                       */
/* =================================                                       */

/* We paint:                                                               */
/*                                                                         */
/* 1. lines that have the character string '.*' or just '*' in column 1    */
/*                                                                         */
/* 2. all characters appearing after the operands of an instruction        */
/*                                                                         */
/* This last is difficult for the editor, that is why we try here.  This   */
/* macro considers one or two special cases:                               */
/*                                                                         */
/* 1.  ANOP and MEND                                                       */
/*     these do not have operands so any other text in the line is taken   */
/*     to be a comment                                                     */
/*                                                                         */
/* 2.  DC C'..'                                                            */
/*     we check to see if there are any blanks in the character string     */
/*                                                                         */
/* 3.  AIF (....).LABEL                                                    */
/*     the text between ( and ) is skipped                                 */
/*                                                                         */
/* 4.  if a single quote ' is found in the operands, the last occurrence   */
/*     of a single quote is searched for and the first non-blank after     */
/*     that is taken to be the comment. This is to trap cases such as      */
/*                         CLC  A,=C' check me '                           */
/*                                                                         */
/* Otherwise we assume that there are no blanks embedded in the mnemonic   */
/* or the intruction's operands.  This will not be true in all cases, for  */
/* example, in CLC =C' string ',X but we apply the law of diminishing      */
/* returns (effort increases disproportionatly for special cases>.         */

continued = false
Do linei = first_line To last_line

  If (linei//100 = 0) Then Do
    'MSG'linei 'lines processed'
  End                                  /* end if                           */

  ''linei
  'EXTRACT /CURLINE'                   /* get contents of current line     */

  If (length(curline.1) = 0) Then Do
    continued = false
    Iterate linei
  End                                  /* end if                           */

  /* o.k,. now try for the trivial case                                    */

  If (left(curline.1,1) = '*') | (left(curline.1,2) = '.*') Then Do
    Call Colour 'COMMENT' 1 Length(curline.1)
    If (substr(curline.1,72,1) <> ' ') Then continued = true
    Iterate linei
  End                                  /* end if                           */

  /* o.k., now for the non-trivial case, try to identify comment           */

  If (continued) Then Do
    mn = 0                             /* no mmemonic                      */
    so = Wordindex(curline.1,1)        /* sm = start of operand            */
    cs = Wordindex(curline.1,2)        /* cs = comment start               */
  End                                  /* end if                           */
  Else If (left(curline.1,1) = ' ') Then Do    /* no label in line         */
    mn = Word(curline.1,1)             /* mn = mnemonic                    */
    so = Wordindex(curline.1,2)        /* sm = start of operand            */
    cs = Wordindex(curline.1,3)        /* cs = comment start               */
  End                                  /* end if                           */
  Else Do                              /* label is present                 */
    mn = Word(curline.1,2)             /* mn = mnemonic                    */
    so = Wordindex(curline.1,3)        /* sm = start of operand            */
    cs = Wordindex(curline.1,4)        /* cs = comment start               */
  End                                  /* end else                         */

  If (cs = 0) Then Do                  /* no comments to paint             */
    Iterate linei
  End                                  /* end if                           */

  /* process one or two special cases                                      */

  Select                               /* select a special case            */

    When (continued) Then Do           /* no need to do anything here,     */
      Nop                              /* this is to avoid entering any    */
    End                                /* of the other Whens.              */

    When (wordpos(mn,'ANOP MEND') <> 0) Then Do
      cs = so                          /* no operands for these mnemonics  */
    End                                /* end when                         */

    When (mn = 'DC') & (substr(curline.1,so,1) = 'C') Then Do

      /* scan forward to first apostrophe                                 */
      Do i = so Until i = Length(curline.1)
        If (substr(curline.1,i,1) = "'") Then Leave
      End                              /* end do                           */

      /* scan forward to last apostrophe                                  */
      Do i = i+1 Until (i = Length(curline.1))
        If (substr(curline.1,i,2) == "' ") Then Leave
      End                              /* end do                           */

      /* point to comment                                                 */
      cs = Verify(curline.1, ' ',, i+1)

    End                                /* end if                           */

    When (mn = 'AIF') & (substr(curline.1,so,1) = '(') Then Do

      'CURSOR' linei '1'

      '/).'                            /* look for closing bracket         */

      'EXTRACT /CURSOR/'
      /* scan for first blank             */
      Do scani = cursor.2+1 To Length(curline.1)
        If (substr(curline.1,scani,1) = ' ') Then Leave
      End                              /* end do                           */

      /* point to comment                                                 */
      cs = Verify(curline.1, ' ',, scani)

    End                                /* end if                           */

    When (pos("'", Word(substr(curline.1,so),1)) <> 0 ) Then Do

      /* scan to find the last quoted string                               */
      'EXTRACT /LINECOLOUR/'

      Do scani = linecolour.0 To 1 By -1
        Parse Var linecolour.scani s_col e_col colcol

        temp = 'QUOTES'
        If (colcol = quotes_area_index)  ,
          |  (Space(colcol) = Space(cols.temp)) Then Do
          Leave scani
        End                            /* end if                           */
      End                              /* end do                           */

      /* point to comment (first non-blank)                               */
      cs = Verify(curline.1, ' ',, e_col+1)
    End                                /* end when                         */

    Otherwise Do
      Nop
    End

  End                                  /* end of select over special cases */

  continued = false                    /* assume cur line not continued    */
  If (substr(curline.1,72,1) <> ' ') Then continued = true

  If (cs < Length(curline.1)) & (cs <> 0) Then Do
    If (continued) Then Do
      Call Colour 'COMMENT' cs 71
      Call Colour non_comm_attr 72 Length(curline.1)
    End                                /* end if                           */
    Else Do
      Call Colour 'COMMENT' cs Length(curline.1)
    End                                /* end else                         */
  End                                  /* end if                           */

End                                    /* end of loop over text lines      */

'TOPLINE' flscreen.1
'CURSOR' cursor.1 cursor.2

Return

/*************************** See to Painting JCL ***************************/
/*            Subprogram to paint comments in a JCL job stream             */
/***************************************************************************/

PAINT_JCL:

/* start loop through lines to paint                                       */
/* =================================                                       */

/* We paint:                                                               */
/*                                                                         */
/* 1. lines that have the character string slash/slash/asterisk            */
/*                                                                         */
/* 2. all characters appearing after the operands of a JCL statement       */
/*                                                                         */
/* This last is difficult for the editor, that is why we try here.  The    */
/* restrict outselves here to JOB, EXEC and DD stmts. The only difficulty  */
/* are JOB and EXEC statemnts which may contain charater strings which     */
/* in turn may contain blanks. This is unfortunate as otherwise blanks act */
/* as delimiters, for example:                                             */
/*                                                                         */
/*    //JOBNAME   JOB  'ACCOUNTING INFO',NOTIFY=ME   comment               */
/*    //STEPNAME  EXEC  PGM=PROG,PARM='PROG PARMS'   another comment       */
/*                                                                         */

continued = false
Do linei = first_line To last_line

  If (linei//100 = 0) Then Do
    'MSG'linei 'lines processed'
  End                                  /* end if                           */

  ''linei
  'EXTRACT /CURLINE'                   /* get contents of current line     */

  If (length(curline.1) = 0) | (left(curline.1,2) <> '//') Then Do
    continued = false                  /* line is empty or not JCL         */
    Iterate linei
  End                                  /* end if                           */

  /* o.k,. now try for the trivial case                                    */

  If (left(curline.1,3) = '//*') Then Do
    Call Colour 'COMMENT' 1 Length(curline.1)
    Iterate linei
  End                                  /* end if                           */

  /* O.k., now for the non-trivial cases. We try to find the start of      */
  /* the operands, and distingush:                                         */
  /*                                                                       */
  /* 1. Normally they will be the third "word":                            */
  /*                                                                       */
  /*       //JOBNAME  JOB  operands                                        */
  /*       //STEPNAME EXEC operands                                        */
  /*       //DDNAME   DD   operands                                        */
  /*                                                                       */
  /* 2. They can be the second word:                                       */
  /*                                                                       */
  /*       //         DD  operands      no ddname                          */
  /*                                                                       */
  /* 3. In the case of continuations, they are also the second word (as    */
  /*    the // are regarded as being the first word):                      */
  /*                                                                       */
  /*       //DDNAME   DD  operands,     continued stmts                    */
  /*       //            operands       continuation                       */
  /*                                                                       */
  /* Having found the operands, we enter a finite state machine to simply  */
  /* skip over quoted strings until we find the first blank (outside a     */
  /* quoted string).  Any characters afer that blank we regard as          */
  /* comments.                                                             */
  /*                                                                       */

  If (continued) Then Do
    so = Wordindex(curline.1,2)        /* sm = start of operand            */
  End                                  /* end if                           */
  Else If (left(curline.1,3) = '// ') Then Do  /* no name in line          */
    so = Wordindex(curline.1,2)        /* so = start of operand            */
  End                                  /* end if                           */
  Else Do                              /* name  is present                 */
    so = Wordindex(curline.1,3)        /* sm = start of operand            */
  End                                  /* end else                         */

  If (so = 0) Then Do                  /* no comments to paint             */
    Iterate linei
  End                                  /* end if                           */

  state = 'OPEN'                       /* in open code                     */

  sq = "'"                             /* single quote                     */
  dq = '"'                             /* double quote                     */

  /* Painting is done by means of a finite state machine. The states,      */
  /* their meanings are:                                                   */
  /*                                                                       */
  /* State  Meaning                     Action                             */
  /*                                                                       */
  /* 'SQ'   single quote detected       Scan for another single quote..    */
  /*                                    (find end of quoted string)        */
  /*                                                                       */
  /* 'DQ'   double quote detected       Scan for another double quote      */
  /*                                    (find end of quoted string)        */
  /*                                                                       */
  /* 'OPEN'  outside quotes            Scan for any of above, and blank    */
  /*                                                                       */

  Do coli = so To Length(curline.1)

    If (state = 'OPEN') Then Do        /* check for end of operands        */
      If (substr(curline.1,coli,1) = ' ') Then Do
        Leave coli
      End                              /* end if                           */
      If (substr(curline.1,coli,1) = sq) Then Do
        state = 'SQ'
      End                              /* end if                           */
      If (substr(curline.1,coli,1) = dq) Then Do
        state = 'DQ'
      End                              /* end if                           */
      Iterate coli
    End                                /* end if                           */

    If (state = 'SQ') Then Do          /* single quote already detected    */
      If (substr(curline.1,coli,1) = sq) Then Do
        state = 'OPEN'
      End                              /* end if                           */
      Iterate coli
    End                                /* end if                           */

    If (state = 'DQ') Then Do          /* double quote already detected    */
      If (substr(curline.1,coli,1) = dq) Then Do
        state = 'OPEN'
      End                              /* end if                           */
      Iterate coli
    End                                /* end if                           */

  End                                  /* end of loop through one line     */

  Call Colour 'COMMENT' coli Length(curline.1)

  If (substr(curline.1,coli-1,2) = ', ') Then Do
    continued = true
  End                                  /* end if                           */
  Else Do
    continued = false
  End                                  /* end else                         */

End                                    /* end of loop throug lines         */

'TOPLINE' flscreen.1                   /* reposition to where we were      */
'CURSOR' cursor.1 cursor.2

Return

/**************** Subprocedure to issue  colouring command *****************/

COLOUR:

Arg c_area c_s c_e

If (c_s > c_e) Then Do                 /* start behind end postion         */
  Return
End                                    /* end if                           */

If (c_area = '?') Then Do
  'LINECOLOUR' c_s c_e cols.non_comm_attr      /* shift in from comment    */
End                                    /* end if                           */
Else Do
  'LINECOLOUR' c_s c_e cols.c_area
End                                    /* end else                         */

If (rc <> 0) Then Do
  'MESSAGEBOX' myname' return code' rc 'from LINECOLOUR, aborting'
  'MESSAGEBOX' myname' c_area, c_s and c_e =' c_area c_s c_e
  Exit 8
End                                    /* end if                           */

Return

/*************************** Debugging Routines ****************************/

Novalue:
'MESSAGEBOX +++ Error in REXX macro' myname '+++' ,
  escape.1'nNovalue condition from line' sigl ,
  escape.1'n'Sourceline(sigl) ,
  escape.1'nVariable:' Condition('D')
Trace ?r; Nop
Exit

Syntax:
'MESSAGEBOX +++ Error in REXX macro' myname '+++' ,
  escape.1'nSyntax error no.' rc 'from line' sigl ,
  escape.1'n'Errortext(rc) ,
  escape.1'n'Sourceline(sigl) ,
  escape.1'nDescription:'Condition('D')
Trace ?r; Nop
Exit

/************************** End of Macro PAINTCOM **************************/
