/***************************************************************************/
/*                                                                         */
/* RECALC.X                                                                */
/*                                                                         */
/* Simple spreadsheet calculations.  Difficult to set up the rules, but    */
/* handy once initialised.  Position the cursor between :data. and :edata. */
/* rows and type "recalc".  Who needs 123/Excel?                           */
/*                                                                         */
/* Specifying data format.  The following tags are REQUIRED:               */
/*                                                                         */
/* :recalc.   - Identifies the beginning of the column formatting rules    */
/* :col       - Defines formatting for a column.  Parameters:              */
/*              name=c1               - name for the column                */
/*              startcol=1|~          - starting column number             */
/*              length=10             - column length                      */
/*              justify=right|none    - output justification               */
/*              datatype=char|number  - type of data                       */
/*              format=n,m            - formatting for numeric data        */
/*              text=xxx              - static text for character data     */
/* :calc_cols - Specifies the order to calculate columns.                  */
/* :formula   - Calculation formula for a column.  Parameters:             */
/*              col=c3                - the column name                    */
/*              calc=100*c8-c2-c8~    - calculation rules                  */
/*              A column name followed by tilde (~) means the previous     */
/*              row's value.  A column name followed by a grave accent (`) */
/*              means the first row's value.                               */
/* :erecalc.  - Identifies the end of the formatting rules                 */
/* :data.     - Identifies the beginning of the data                       */
/* :edata.    - Identifies the end of the data                             */
/*                                                                         */
/*                                                                         */
/* Example.  We have bought 200 shares of an American stock on May 1st, at */
/* $10 per share.  The stock is now worth $12.36 a share, we want to       */
/* calculate the value in Canadian dollars and our percent return.         */
/*                                                                         */
/* :recalc.                                                                     */
/* :col name=c1  startcol=1 length=10 justify=right datatype=char.              */
/* :col name=c2  startcol=~ length=11 justify=right datatype=number format=4,4. */
/* :col name=c4  startcol=~ length=8  justify=right datatype=number format=5,2. */
/* :col name=c6  startcol=~ length=8  justify=right datatype=number format=5,2. */
/* :col name=c5  startcol=~ length=3  justify=right datatype=char   text=$.     */
/* :col name=c7  startcol=~ length=10 justify=right datatype=number format=5,2. */
/* :col name=c8  startcol=~ length=7  justify=right datatype=number format=3,2. */
/* :col name=c9  startcol=~ length=4  justify=left  datatype=char   text=%.     */
/* :col name=c10 startcol=~ length=20 justify=left  datatype=char.              */
/* :calc_cols c6 c4 c7 c8.                                                      */
/* :formula col=c4 calc=c6-c6~                                                  */
/* :if col=c6 ifvalue=0 calc=c6~*c2/c2~                                         */
/* :formula col=c7 calc=c6*1.4800                                               */
/* :formula col=c8 calc=c7-c7`*100/c7`                                          */
/* :erecalc.                                                                    */
/*                                                                              */
/*    Date      Unit Val  Growth    Value      CAD     Return                   */
/* :data.                                                                       */
/*   00/05/01    10.0000 2000.00 2000.00  $   2960.00   0.00%                   */
/*   00/06/01    12.3600  472.00 2472.00  $   3658.56  23.60%                   */
/* :edata.                                                                      */
/*                                                                         */
/*                                                                         */
/* Written by Blair Thompson, March 5, 1997                                */
/* Add tilde to startcol specification, May 26, 1997                       */
/* Add backquote (`) to indicate 1st row, March 1, 2000                    */
/* Add example, June 12, 2000                                              */
/* Add linefields to simplify tabbing through the data, June 21, 2000      */
/* Allow parentheses, December 10, 2000                                    */
/* Add :total_cols & :subtotal tags, December 28, 2000                     */
/*                                                                         */
/***************************************************************************/

  'EXTRACT /CURSOR/'
  'L /:recalc./ -c'
  lrc = rc
  'RESTORE_FIND'
  If lrc <> 0
    Then Do
      'MSG :recalc. tag not found'
      Exit 99
    End

  data. = ''
  data._curcol = 0                                /* Starting column number*/

  Call read_defs
  If rc <> 0
    Then Exit rc

  data._maxcol = data._curcol

  'CURSOR' cursor.1 cursor.2
  Call process_data
  'CURSOR' cursor.1 cursor.2
Exit


/***************************************************************************/
/*                                                                         */
/* Read & save total_cols, December 28, 2000                               */
/*                                                                         */
/***************************************************************************/

READ_DEFS:  Procedure Expose data.
  Do Forever
    'DOWN 1'
    If rc <> 0
      Then Do                                                        /* EOF*/
        'MSG EOF reached before end of definition lines'
        Return 99
      End

    'EXTRACT /CURLINE/'
    If Substr(curline.1,1,1) <> ':'
      Then Do                                          /* Unrecognised line*/
        'MSG Unrecognised line'
        Return 99
      End

    Parse Upper Var curline.1 ':' tag parms
    Select
      When tag = 'COL'
        Then Do
          data._curcol = data._curcol + 1
          varname = data._curcol
          Parse Var parms parms '.' .
          Parse Var parms prefix 'STARTCOL=' scol suffix
          If scol = '~'
            Then Do                   /* Calculate column based on previous*/
              If varname = 1
                Then scol = 1                       /* Default start column*/
                Else Do                        /* We have a previous column*/
                  col = varname - 1
                  Parse Var data.col . 'STARTCOL=' scol .
                  Parse Var data.col . 'LENGTH=' len .
                  scol = scol + len
                End                                               /* End do*/
              parms = prefix 'STARTCOL='scol suffix
            End                                                   /* End do*/
          data.varname = parms
        End
      When tag = 'CALC_COLS'
        Then Parse Var parms data._calc_cols '.' .
      When tag = 'TOTAL_COLS'
        Then Parse Var parms data._total_cols '.' .
      When tag = 'FORMULA'
        Then Call process_formula parms
      When tag = 'IF'
        Then Call process_if parms
      When tag = 'ERECALC.'
        Then Leave
      Otherwise Do
        'MSG Unrecognised tag:' tag
        Return 99
      End
    End                                                       /* End select*/
  End                                                             /* End do*/
Return 0


/***************************************************************************/
/*                                                                         */
/* We read an :if line.  The expected format is                            */
/*                                                                         */
/*    :if col=c ifvalue=x calc=....                                        */
/*                                                                         */
/***************************************************************************/

PROCESS_IF:  Procedure Expose data.
  Parse Arg parms
  colnum = 0
  Do While parms <> ''
    Parse Var parms parm parms
    Parse Var parm opt '=' setting
    Select
      When opt = 'COL'
        Then colnum = find_col(setting)
      When opt = 'IFVALUE' & colnum > 0
        Then data.colnum = data.colnum parm
      When opt = 'CALC' & colnum > 0
        Then data.colnum = data.colnum parm
      Otherwise Nop
    End                                                       /* End select*/
  End
Return


PROCESS_FORMULA:  Procedure Expose data.
  Parse Arg parms
  colnum = 0
  Do While parms <> ''
    Parse Var parms parm parms
    Parse Var parm opt '=' setting
    Select
      When opt = 'COL'
        Then colnum = find_col(setting)
      When opt = 'CALC' & colnum > 0
        Then data.colnum = data.colnum parm
      Otherwise Nop
    End                                                       /* End select*/
  End
Return


/***************************************************************************/
/*                                                                         */
/* Translate a column name into a column number                            */
/*                                                                         */
/***************************************************************************/

FIND_COL:  Procedure Expose data. previous.
  Parse Arg targ_col .

  Do col = 1 To data._curcol
    Call get_colinfo data.col
    If col._name = targ_col
      Then Return col
  End                                                             /* End do*/
Return 0                                                /* Column not found*/


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

GET_COLINFO:  Procedure Expose col.
  Parse Arg data
  Parse Value '' With col._name     col._startcol col._length col._text,
                      col._ifvalue  col._justify  col._fmt1   col._fmt2,
                      col._datatype col._calc
  Do While data <> ''
    Parse Var data parm data
    Parse Var parm opt '=' setting
    Select
      When opt = 'NAME'
        Then col._name = setting
      When opt = 'STARTCOL'
        Then col._startcol = setting
      When opt = 'LENGTH'
        Then col._length = setting
      When opt = 'JUSTIFY'
        Then col._justify = setting
      When opt = 'FORMAT'
        Then Parse Var setting col._fmt1 ',' col._fmt2
      When opt = 'DATATYPE'
        Then col._datatype = setting
      When opt = 'CALC'
        Then col._calc = setting
      When opt = 'IFVALUE'
        Then col._ifvalue = setting
      When opt = 'TEXT'
        Then col._text = setting
      Otherwise Nop
    End                                                       /* End select*/
  End                                                             /* End do*/

Return


/***************************************************************************/
/*                                                                         */
/* Apply our formatting rules to each line of data between :data. and      */
/* :edata.                                                                 */
/*                                                                         */
/* Add totals if necessary, December 28, 2000                              */
/*                                                                         */
/***************************************************************************/

PROCESS_DATA:  Procedure Expose data.
  'L /:edata./ c'
  lrc = rc
  'RESTORE_FIND'
  If lrc <> 0
    Then Do
      'MSG Unable to find :edata. tag'
      Return
    End

  'L /:data./ -c'
  lrc = rc
  'RESTORE_FIND'
  If lrc <> 0
    Then Do
      'MSG Unable to find :data. tag'
      Return
    End

  contents. = 0
  total.    = 0
  subtotal. = 0
  Do rownum = 1
    'DOWN 1'
    If rc <> 0
      Then Leave                                                     /* EOF*/
    'EXTRACT /CURLINE/'
    w1 = Word(translate(curline.1),1)
    If w1 = ':EDATA.' | w1 = ':TOTAL.'
      Then Leave                                             /* End of data*/
    If w1 = ':SUBTOTAL.'
      Then Do
        Call input_total ':subtotal.' 'SUBTOTAL'
        Do colnum = 1 To data._maxcol                /* Reset the subtotals*/
          subtotal.colnum = 0
        End                                                       /* End do*/
      End                                                         /* End do*/
      Else Do                                    /* Regular line processing*/
        Do colnum = 1 To data._maxcol
          previous.colnum = contents.colnum
          contents.colnum = 0
        End                                                       /* End do*/
        Call process_line curline.1
      End
  End                                                             /* End do*/

  If data._total_cols <> ''
    Then Do
      If w1 = ':EDATA.'
        Then Do
          'UP 1'
          'INPUT'
        End
      Call input_total ':total.' 'TOTAL'
    End                                                           /* End do*/
Return                                      /* End of Process_data function*/


/***************************************************************************/
/*                                                                         */
/* Input the total so far, with the specified identifier.                  */
/*                                                                         */
/* Written by B. Thompson, December 28, 2000                               */
/*                                                                         */
/***************************************************************************/

INPUT_TOTAL:  Procedure Expose data. total. subtotal.
  Parse Arg identifier varname .
  outline = Left(identifier,100)
  tcols = data._total_cols
  Do While tcols <> ''
    Parse Var tcols colname tcols
    colnum = find_col(colname)
    Call get_colinfo data.colnum
    coldata = Value(varname'.colnum')
    If col._justify = 'RIGHT'
      Then coldata = Right(coldata,col._length)
    outline = Overlay(coldata,outline,col._startcol,col._length)
  End                                                             /* End do*/

  'extract /curline/'
  If outline <> curline.1
    Then 'REPLACE' outline
Return                                       /* End of Input_total function*/


/***************************************************************************/
/*                                                                         */
/* Apply formatting rules to the current line                              */
/*                                                                         */
/***************************************************************************/

PROCESS_LINE:  Procedure Expose data. contents. previous. ,
                                first. rownum total. subtotal.
  Parse Arg linedata
  outline = Left('',100)
  calc_cols = ''
  fields = ''
  Do col = 1 To data._maxcol
    Call get_colinfo data.col
    fields = fields col._startcol 'P' col._startcol + 1 'U'
    If col._calc = ''
      Then Call format_col col linedata
      Else calc_cols = calc_cols col._name
    If rownum = 1
      Then first.col = contents.col               /* Store first row's data*/
  End                                                             /* End do*/

  If data._calc_cols <> ''
    Then calc_cols = data._calc_cols

  Do While calc_cols <> ''
    Parse Var calc_cols col calc_cols
    colnum = find_col(col)
    Call get_colinfo data.colnum
    Call format_col colnum linedata
    If rownum = 1
      Then first.colnum = contents.colnum         /* Store first row's data*/
  End                                                             /* End do*/
  If outline <> linedata
    Then 'REPLACE' outline

/*
  'linefields' fields
*/
Return


/***************************************************************************/
/*                                                                         */
/* Calculate a column according to specified rules.                        */
/*                                                                         */
/***************************************************************************/

CALC_COL:  Procedure Expose data. contents. previous. first.
  Parse Arg calc .

  Parse Value '0 1 0 0' With result add mult div prefix suffix .
  If Pos('(',calc) > 0
    Then Parse Var calc prefix '(' calc ')' suffix .
  Do While calc <> ''
    plus   = pos('+',calc); If plus   = 0 Then plus   = 100
    minus  = pos('-',calc); If minus  = 0 Then minus  = 100
    times  = pos('*',calc); If times  = 0 Then times  = 100
    divide = pos('/',calc); If divide = 0 Then divide = 100
    sign = Min(plus,minus,times,divide)

    If sign = 100
      Then Do                                                   /* Last one*/
        col = calc
        calc = ''
      End
      Else Do              /* We have a mathematical operation to parse out*/
        col = Substr(calc,1,sign-1)
        calc = Substr(calc,sign+1)
      End

    Select
      When Pos('~',col) > 0
        Then Do                                             /* Previous row*/
          Parse Var col col '~' .
          col = find_col(col)
          col = previous.col
        End
      When Pos('`',col) > 0
        Then Do                                                  /* 1st row*/
          Parse Var col col '`' .
          col = find_col(col)
          col = first.col
        End
      When Datatype(col,'S')
        Then Do
          val = find_col(col)
          If val > 0
            Then col = contents.val
        End
      Otherwise Nop
    End                                                       /* End select*/

    Select
      When \Datatype(col,'N')
        Then Do
          'MSG Non-numeric column data' col
          Exit
        End
      When add <> 0
        Then result = result + (add * col)
      When mult
        Then result = result * col
      When div
        Then Do                              /* Division takes special care*/
          If col = 0
            Then result = 0
            Else result = result / col
        End
      Otherwise Nop
    End                                                       /* End select*/

    Select
      When sign = plus
        Then Parse Value '1 0 0' With add mult div .
      When sign = minus
        Then Parse Value '-1 0 0' With add mult div .
      When sign = times
        Then Parse Value '0 1 0' With add mult div .
      When sign = divide
        Then Parse Value '0 0 1' With add mult div .
      Otherwise Nop
    End                                                       /* End select*/

    If calc = '' & prefix <> ''
      Then Do                                          /* In case of parens*/
        calc = prefix || result || suffix
        Parse Value '0 1 0 0' With result add mult div prefix suffix .
      End                                                         /* End do*/
  End                                                             /* End do*/
Return result


/***************************************************************************/
/*                                                                         */
/* Format the specified column number according to the rules we already    */
/* read.                                                                   */
/*                                                                         */
/***************************************************************************/

FORMAT_COL:  Procedure Expose data. contents. previous. ,
                              col. outline first. total. subtotal.
  Parse Arg col linedata
  coldata = Substr(linedata,col._startcol,col._length)
  If col._justify <> 'NONE'
    Then coldata = Strip(coldata)
  If col._datatype = 'NUMBER'
    Then Do                                              /* Verify the data*/
      If coldata = '' | \Datatype(coldata,'N')
        Then coldata = 0
    End
  If col._text <> ''
    Then coldata = col._text                               /* Constant text*/
  If col._calc <> ''
    Then Do                                  /* Maybe calculate this column*/
      If col._ifvalue = '' | coldata = col._ifvalue
        Then coldata = calc_col(col._calc)
    End
  If col._fmt1 <> '' & col._fmt2 <> ''
    Then Do                                     /* Check for maximum length*/
      val = coldata % 1
      If Length(val) > col._fmt1
        Then Do                                                   /* Error!*/
          'MSG Calculated value of' val 'is too long to fit in supplied Format'
          Exit
        End
      coldata = Format(coldata,col._fmt1,col._fmt2)
    End
  contents.col = coldata
  If col._datatype = 'NUMBER'
    Then Do                                           /* In case we need em*/
      total.col = total.col + coldata
      subtotal.col = subtotal.col + coldata
    End                                                           /* End do*/
  If col._justify = 'RIGHT'
    Then coldata = Right(coldata,col._length)
  outline = Overlay(coldata,outline,col._startcol,col._length)
Return                                        /* End of Format_col function*/
