/***************************************************************************/
/*  REXXFMT - X Editor  macro to reformat REXX execs                       */
/***************************************************************************/
/*

         REXXFMT - Format a REXX Program to your Specifications
         ======================================================

 This Editor macro  reformats  REXX  EXECs.   You  can  specify the  case  of
 keywords, functions, and  labels, and specify the number of  blank spaces to
 indent DO groups.  You can optionally split lines at semi-colons and at THEN
 statements.

 If a mark exists, the lines within the  mark are formatted.  If a  mark does
 not exist, the whole file is formatted.  Only line marks are accepted.

   
     REXXFMT [ options ]                                                 
                                                                         
     where    options are either ? (to put out help text) or one or      
                      more of:                                           
                                                                         
                                                                         
                 Noprof                                                  
                 Indent      nn                                          
                 Leftmargin  lm                                          
                 Lm          lm                                          
                 Rightmargin rm                                          
                 Rm          rm                                          
                                                                         
     each option may be arbitrarily abbreviated down to the capital      
     letter.                                                             
                                                                         
   ΄

 where

  ?     Instructs this macro to put out these comments as  a  primitvee  help
        facility. This option is mutually exclusive to all other options.

  Noprof Instructs this macro not to use the profile file as defined  in  the
        varible pro.fname (see procedure read_profile). This option may  also
        be coded /NOPROF,  -NOPROF. Any abbreviation down to N, /N or -N will
        be accepted.

  nn    The number of spaces to indent loops.  The default is 2.

  lm    Create  left  margin  such  that the first non-comment line begins in
        column 'lm'. Useful when REXXFMT'ing subsections of  a  program.  The
        default value is 3. If a value of 0 is given, this macro searches for
        the first non-blank character in the  text  to be formatted, and uses
        its column number as the left margin.

  rm    Column  for  right-aligning  comments.  Default  is  72. Specifying 0
        results in using the current value of the right margins setting.

  Implementation Features:
  ------------------------

 Lines containing 'DO' or 'SELECT' are indented, while lines containing 'END'
 are  un-indented.   Continuation  lines  are indented  additional  space(s).
 Indenting is suppressed for lines containing a label.  Optionally, all lines
 containing THEN statements can  be split so the THEN always  starts on a new
 line.  Lines beginning with THEN and ELSE are also indented.

 Lines beginning with '/*' can be either left alone or indented.  The default
 is to indent.   Where possible, lines which end in a  comment ('*/') will be
 adjusted so that the '/' will fall in column 72 (rightmargin).

 If the  modified line exceeds  the LRECL of the  original file, the  line is
 left unmodified.

 Keywords are  not scanned exactly  as the  language specifies.   'FUZZ', for
 example, is always considered a keyword, not just when it follows 'NUMERIC'.
 Keywords are checked for last, in case a label or function has the same name
 as a keyword.

 Comments and literal strings are left unchanged.

 You may override many of the default  values  for  case  settings  and  code
 modifications.  A  profile  file  is  used  called  REXXFMT.PRO.  This  file
 conains settings for a number of options, these  are  described  below.  The
 values  from  the profile file override the hard-coded values from the exec,
 the options specified on the invocation line as described above override the
 values  taken  from  the  profile file. If you wish to change the hard-coded
 values, please check the  subprocedure  READ_PROFILE.  This  procedure  also
 contains  the  name,  including  the full path specification, of the profile
 file which you almost certainly will want to change. Needless to  say,  this
 macro has processed itself using the hard-coded options.

  Acknowledgements
  ----------------

 This code comes from  a VMSHARE append which included an  XEDIT macro called
 REXXREF,  which  performed  the  REXXFMT function  plus  generated  a  cross
 reference listing.  The user configuration code was added and a good deal of
 modifications  were  made  to  the  code  in  the  name  of  efficiency  and
 configurability.

  Using the profile file
  ----------------------

 This  macro  reads  a  profile  file  whose name is (under OS/2 and Windows)
 derived  from  the  contents  of  the   environment   variable   XPATH   and
 'REXXMFT.PRO'.  If  you  wish to use another name, please check the variable
 pro.fname in subprocedure READ_PROFILE.

 The REXXFMT.PRO file contains all options that can be used to control
 the formatting of your REXX code.  The following labels can be specified:

    cont_indent     - contains the number of spaces to indent lines that
                      are continued (i.e. end in a comma)
    indent          - the number of spaces to indent new loops etc.
    variable_case   - the case for all variables.  Options are U, M, and
                      L for UPPERCASE, Mixed case, and lower case
                      respectively.
    label_case      - the case for all labels.  Options are U, M, and L
                      and have the same meanings as above.
    label_call_case - the case for all calls to labels.  Options are U,
                      M, and L and have the same meanings as above.
    keyword_case    - the case for all REXX keywords.  Options are U, M,
                      and L and have the same meanings as above.
    function_case   - the case for all internal REXX functions.  Options
                      are U, M, and L and have the same meanings as
                      above.
    lm              - the left margin for starting indentation. If 0, the
                      column of the first non-blank character in the text
                      to be formatted is used.
    rm              - the right margin for the end of comments. If 0, the
                      current value of the right margins setting is used.
    cmt_rjust       - A flag telling whether or not to try to right
                      adjust comments to the right margin.  The default
                      is 1 (yes).
    cmt_indent      - A flag specifying whether or not to indent single
                      line comments at the current margin.  The default
                      is 1 (yes)
    split_then      - A flag specifying whether or not to split an IF
                      statement at the THEN statement.  If 1 (on - the
                      default), all THEN statements will be indented to
                      begin on a new line.  If 0, lines will not be split
                      at THEN statements.
    split_after_then - A flag specifying whether or not to split a line
                      so that the clause following a THEN or ELSE
                      statement should be formatted to a new line.  The
                      default value, 0, means leave the line as is.  If
                      set to 1, this option will split lines after THEN
                      statements.  Note that this option in conjunction
                      with the SPLIT_THEN option will force THEN and ELSE
                      statements to occupy a single line.
    semicolon       - Controls whether to split lines containing
                      semi-colons (;) into separate lines.  The default
                      value is 1, which means to split the lines
    noindent        - A list of keywords to be forced flush left if found
                      at the beginning of the line.
    refresh         - Not used. Is recognised (but not used here) for
                      compatibility with CMS.
    endindent       - A flag indicating whether or not to indent END
                      statements so they line up with the code, or are
                      indented with the rest of the DO/SELECT block.  The
                      default value, 0, means do not indent END
                      statements, i.e. line them up with the line that
                      contains the preceding DO.
    indent_threshold - A number indicating a column, after which the
                      INDENT and CONT_INDENT variables automatically
                      revert to 1.  The purpose of this option is to
                      allow heavily nested code a better chance of
                      remaining on a single page.  The default value is
                      20.

 The following lines illustrate the original format for specifying
 these options, this format is derived from the CMS NAMES facility
 and has been retained for compatibility.

    * comments are writtten..
    * ..with an asterisk in col. 1
    :nick.REXXFMT
         :cont_indent.2
         :indent.2
         :variable_case.L
         :label_case.U
         :label_call_case.L
         :keyword_case.M
         :function_case.M
         :lm.3
         :rm.72
         :cmt_rjust.1
         :cmt_indent.1
         :split_then.1
         :split_after_then.0
         :semicolon.1
         :noindent.EXIT RETURN
         :refresh.M
         :endindent.0
         :indent_threshold.20

 This macro ignores the :nick. line, any blank lines and any lines that
 start with '*' (or the REXX opening comment charcaters '/'||'*').  In
 addition to the above format, the following format will also be
 accepted (i.e. with '=' and without ':' and '.', blanks are discarded)

        cont_indent = 2
        variable_case = L

 etc. Both formats may be intermixed in the profile file. In addition, the
 'flags' options which may be given as 0 or 1 may be given as
 true or false, yes or no:

        split_then = false
        split_after_then = true

  -------------------------------------------------------------------------*/

  Signal endhelpline                /* these two lines define the ...,*/
ENDHELPLINE: lasthelpline = sigl -1             /* ...text for 'help' */

/*-------------------------------------------------------------------------*/
/* N O T E !  keep the above two lines as they are used to pick up         */
/* the number of the last line of help text to put out to a window.        */
/*-------------------------------------------------------------------------*/

/**********************************************************************/
/*                                                                    */
/* No other routines are called.                                      */
/*                                                                    */
/* Written by Don Hughes.  June, 1985                                 */
/*            EDP Techniques                                          */
/*            WHITE PLAINS, NY                                        */
/*            (914) 949-5994                                          */
/*                                                                    */
/* Update log:                                                        */
/*                                                                    */
/* Date       By        Comments                                      */
/*  6 Dec 85  Don Hughes Submitted to VMSHARE                         */
/* ?? ??  86  HENDERS   Change to Waterloo up/low case conventions.   */
/* 16 Jan 87  HENDERS   Do label check before keyword check; set      */
/*                      SPILL OFF for main loop; only type XREF when  */
/*                      requested.                                    */
/* 19 Jan 87  HENDERS   Split all '/*' and '*/' character strings,    */
/*                      use symbolic references; also for '' and ""   */
/* 21 Jan 87  HENDERS   Indent continuation lines by 2 spaces.        */
/* 12 May 87  HENDERS   Do not change indentation of comment lines.   */
/* 13 May 87  HENDERS   Fix sorting of cross-reference; change        */
/*                      comments to mixed case;                       */
/*                      invoke REXXREF on itself.                     */
/* 14 May 87  HENDERS   Issue a REFRESH at end of main loop; add      */
/*                      diag, diagrc, queued, watrxf to funct list.   */
/* 15 May 87  HENDERS   Recognize 'signal on|off|value token';        */
/*                      move variable assignments out of SCANTOKEN    */
/*                      and FUNCTION.                                 */
/* 18 May 87  HENDERS   Ensure that SORT routine maintains order;     */
/*                      change 'right align comment' philosophy.      */
/* 19 May 87  HENDERS   Re-work 'target' handling.                    */
/* 20 May 87  HENDERS   Change option parsing and add new ones.       */
/*  6 Jul 87  HENDERS   ALT count reflects actual number of changes.  */
/*                      implement 'Leftmargin' option.                */
/*  9 Jul 87  HENDERS   Fix indentation on continuation lines where   */
/*                      a comment follows the continuation ','.       */
/*  4 May 88  HENDERS@WATDCS Identify lines containing '\' characters */
/* 24 May 1988 Copied from VMSHARE by Blair Thompson                  */
/*             Removed DEBUG, COLIN, and XREF options                 */
/*  8 Jun 1988 Allow SPLIT_THEN profile option                        */
/*             Add SEMICOLON profile option                           */
/* 18 Jun 1988 Several bug fixes, add label_call_case and             */
/*             refresh profile options                                */
/* 19 Jun 1988 Incorporate VARCURL simulation from VMSHARE            */
/* 21 Jun 1988 Incorporate some performance improvements from VMSHARE */
/* 22 Jun 1988 Improved error handling routine from Les Koehler       */
/*             Correct indentation when "THEN" and "DO" are on        */
/*             separate lines                                         */
/*  5 Jul 1988 Initial VMTOOLS release                          (0.1) */
/*             Fix bug with no RESERVED lines                   (0.11)*/
/*  6 Jul 1988 Do not adjust comments that remain open          (0.12)*/
/*             Add ENDINDENT option                                   */
/*  7 Jul 1988 Add INDENT_THRESHOLD option                      (0.2) */
/*             Fix semi-colon and OTHERWISE bugs                      */
/*             Align ELSE with previous THEN line (bug fix)           */
/*  8 Jul 1988 Fix problem with multiply nested THEN and ELSE's       */
/*             Blank out semi-colons that are followed by comments    */
/*             Add SPLIT_AFTER_THEN option                            */
/*             Implement REFRESH M option                       (0.3) */
/* 11 Jul 1988 Correct NOVALUE bug caused by improper nesting   (0.31)*/
/* 12 Jul 1988 Move CURLINE down to M before stopping                 */
/*             Fix indentation bug when SPLIT_THEN = 0                */
/* 13 Jul 1988 Fix continue indent bug with function calls            */
/*             Fix nested THEN indentation bug                  (0.32)*/
/* 31 Jan 2000 Roger Spencer: ported to Blair's X editor              */
/*             CMS bifs removed, ObjREXX bifs added, find( replaced   */
/*             by wordpos(, 0 for lm and rm, BROWSE check etc.        */
/*                                                                    */
/**********************************************************************/

  Signal On NOVALUE
  Signal On SYNTAX

/* Define in the variable 'filespec_os' the names of systems as       */
/* returned by 'EXTRACT /OS/' that support the filespec() function.   */

  filespec_os = 'OS/2 WINDOWS NT WINDOWS 95'

  'EXTRACT /OS/'
  If (Wordpos(Translate(os.1), filespec_os) = 0)
    Then Do
      myname = 'REXXFMT'/* filespec not supported, use hardcoded value */
    End                                                     /* end if */
    Else Do
      Parse Source . . myname .
      myname = Translate(Filespec('Name', myname))
      Parse Var myname myname '.' .                 /* drop extension */
    End                                                     /* end if */

/*************** Pick up option, see if help requested ****************/

  Arg parm         /* assume opts to be passed to editor at invocation*/

  'EXTRACT /FLSCREEN/'            /* get no. of lines on screen       */
  If (flscreen.0 <> 2)
    Then window_size = 18                 /* set up no. of lines...   */
    Else Do                       /* ...for pop-up windows            */
      window_size = (flscreen.2 - flscreen.1 + 1)- 4
    End

  help = '? /? -? HELP'     /* if any of these passed at invocation...*/
  If (Wordpos(parm, help) <>0)
    Then Do                                   /*...put out comments.. */
      window_width = Length(Sourceline(1))+1    /* .. as help text    */
      'WINDOW' window_size window_width lasthelpline myname 'Help text'
      Do i = 1 To lasthelpline-1  /* make sure loop ends sometime     */
        'WINLINE' Sourceline(i) || '\nREFRESH'
      End
      Exit 0
    End

/******************* check if we are in browse mode *******************/

  "EXTRACT /FILEINFO/"
  If (fileinfo.5 = 'BROWSE')
    Then Do
      "MESSAGEBOX" myname": In Browse mode, exiting"
      Exit 0
    End                                                     /* end if */

/**************** Set various variables and switches ******************/
  ?rc = 0            /* Holds return code to be passed back to system */
  com.squote = "'"                             /* Character constants */
  com.dquote = '"'
  com.astslash = '*' || '/'
  com.slashast = '/' || '*'
  com.semicolon = ';'

/** 'keywords.scantok' & 'keywords.primary' are used in 'SCANTOKEN', **/
/************* but defined here for execution efficiency. *************/
  keywords.scantok = 'address arg arg by call digits do drop else' ,
    'end engineering error exit expose external for forever form fuzz' ,
    'halt if interpret iterate leave nop novalue numeric off on' ,
    'otherwise parse procedure pull push queue return say scientific' ,
    'select signal source syntax then to trace until upper value var' ,
    'when while with'

/*********** 'keywords.func' is used in routine 'FUNCTION' ************/
/************** this list includes ObjREXX specific bifs **************/
/*************** but does not include the RexxUtil bifs ***************/
/**************** i.e. the ones named RX... or SYS... *****************/
  keywords.func = 'ABBREV ABS ADDRESS ARG BEEP BITAND BITOR BITXOR' ,
    'B2X CENTER CENTRE CHANGESTR CHARIN CHAROUT CHARS' ,
    'COMPARE CONDITION COPIES COUNTSTR C2D C2X' ,
    'DATATYPE DATE DELSTR DELWORD DIRECTORY D2C D2X ERRORTEXT' ,
    'FILESPEC FORM FORMAT FUZZ INSERT LASTPOS' ,
    'LEFT LENGTH LINEIN LINEOUT LINES MAX MIN OVERLAY POS' ,
    'QUEUED RANDOM REVERSE RIGHT SIGN SOURCELINE' ,
    'SPACE STREAM STRIP SUBSTR SUBWORD SYMBOL TIME' ,
    'TRACE TRANSLATE TRUNC VALUE VAR VERIFY WORD' ,
    'WORDINDEX WORDLENGTH WORDPOS WORDS XRANGE X2B X2C X2D'

/******* Here are the switches that control the logic flow ********/

  switch. = 0                              /* Initialise all switches */

/************************* Search parameters **************************/
  search_key = ''
  searchkey_cnt. = 0                   /* Nesting level of delimiters */
  searchkey_list. = ''            /* Line-number stack for delimiters */
  refresh_ctr = 1                    /* Iterations since last refresh */

/******* 'com.' variables are also passed to called functions *********/
  com.tokennum = 0                            /* Current token number */
  com.last_key_num = 0                /* Token number of last keyword */
  com.last_token = ''                               /* Previous token */
  com.dolist = ''         /* line-number stack for DO's and SELECTS's */
  com.thendolist = ''                  /* Stack for THEN DO / ELSE DO */
  com.doindent = ''              /* List of previous DO indent levels */
  com.next_offset_list = ''
  com.endlist = ''                     /* Line-number stack for END's */
  com.delim = ''                                 /* Current delimiter */
  com.last_char = ''         /* Last non-blank character on each line */
  com.thenelse_nest = 0   /* Number of consecutive THEN or ELSE lines */
  com.prev_thenelse = ''                     /* Previous THEN or ELSE */
  com.select_line = 0                          /* Current SELECT line */

  Arg args                                   /* Check input arguments */

/*********** see if the profile file is to be suppressed **************/

  noprof = 0                        /* false, we want to use the file */
  Do profi = 1 To Words(args)

    currarg = Word(args,profi)

    If (Abbrev('NOPROF',currarg),
      |   Abbrev('/NOPROF',currarg),
      |   Abbrev('-NOPROF',currarg)  )
        Then Do

          noprof = 1   /* true, we don't want to use the profile file */

          args = Delword(args, profi, 1)      /* delete noprof option */

        End                                                 /* end if */

  End                                                       /* end do */

/*********** now set up options, perhaps with profile file ************/

  Call read_profile                            /* Check for a profile */

  Call check_argstring args           /* here to overwrite file input */

  com.offset = pro.lm - 1                  /* Offset for current line */
  com.next_offset = com.offset                /* Offset for next line */
  com.last_then_offset = '0:'com.offset
  com.base_offset = com.offset
  com.prev_then = 0

/*********************** Get the range of lines ************************/

  'EXTRACT /MARK/'
  If (mark.0 = 0)
    Then Do                                         /* no marked area */
      'EXTRACT /FILEINFO/'
      first_line = 1                          /* start at top of file */
      last_line  = fileinfo.2
    End                                                     /* end if */
    Else Do
      If (mark.4 = 0)
        Then Do                                   /* it's a line mark */
          first_line = mark.2
          last_line  = mark.3
        End                                                 /* end if */
        Else Do            /* it's a block mark, we don't suport that */
          'MESSAGEBOX Not a line mark, aborting'
          Exit 0
        End                                               /* end else */
    End                                                   /* end else */

/*************** check the left and right margins value ***************/

  If (pro.lm = 0)
    Then Do
      Do com.linecnt = first_line To last_line
        'EXTRACT /LINE' com.linecnt
        pro.lm = Verify(line.1, ' ')            /* get first nonblank */
        If (pro.lm <> 0)
          Then Leave
      End                                                   /* end do */
    End                                                     /* end if */
  If (pro.lm = 0)
    Then pro.lm = 1                        /* if not found, use col 1 */

  If (pro.rm = 0)
    Then Do
      'EXTRACT /MARGINS/'
      pro.rm = margins.4      /* Right margin for justifying comments */
    End                                                     /* end if */

/*********************** Get some editor settings **********************/
  'LINEND OFF'
  'EXTRACT /FLSCREEN/'
  'EXTRACT /CURLINE/'
  curlmin = flscreen.1                /* first screen line we can use */
  curlmax = flscreen.2                 /* Last screen line we can use */

  curll = flscreen.1
  curltarg = 0

  cursor.1 = 0                                    /* to avoid novalue */
  cursor.2 = 0                                    /* to avoid novalue */
  'EXTRACT /CURSOR/'
  start_line = cursor.1

  'CURSOR' curll

/****************** This is the main processing loop *******************/

  Do com.linecnt = first_line
    If com.linecnt > last_line
      Then Leave                                       /* End of loop */
    ''com.linecnt
    'EXTRACT /CURLINE'
    com.raw_line = curline.1
    If com.raw_line = '' & curll \= curltarg
      Then Do
        curll = curll + 1
        Iterate com.linecnt
      End
    old_len = Length(com.raw_line)
    If \switch.search
      Then s = Strip(com.raw_line)
      Else s = com.raw_line
    com.num_leading_blanks = old_len - Length(s)
    out_string = ''                      /* Work area for output line */
    token = ''                            /* Work area for next token */
    com.delim = ''                               /* Current delimiter */

    Select
      When switch.split
        Then switch.split = 0                  /* Last line was split */
      When switch.in_comment
        Then Nop                           /* Leave indentation alone */
      Otherwise Do
        com.prev_offset = com.offset        /* Previous line's offset */
        test_offset = move_offset(pro.lm-1,pro.indent,Words(com.dolist))
        com.offset = Max(com.next_offset,pro.lm-1,test_offset)
        If switch.reset_indent
          Then Do                 /* Re-set indentation for next line */
            switch.reset_indent = 0
            com.next_offset = move_offset(com.next_offset,pro.indent,-2)
          End
          Else com.next_offset = com.offset
        com.save_offset = com.prev_offset com.offset com.next_offset
      End
    End

    Call scan_line                        /* Check for special tokens */

    If switch.continue
      Then Do                                     /* Continuation line*/
        com.next_offset = com.offset
        If switch.last_continue      /* Was last line continued also? */
          Then com.offset = com.prev_offset
          Else com.offset = move_offset(com.prev_offset,pro.cont_indent,1)
        switch.last_continue = 1
      End

    /* No further indenting if still searching for ending delimiter **/
    If switch.search
      Then switch.skip = 1

    If switch.sc
      Then Do                             /* Split line at semi-colon */
        switch.sc = 0
        Call split_a_line com.sc_col-1 com.sc_col+1
        Iterate com.linecnt
      End

    If switch.split_then
      Then Do                         /* Split line at THEN statement */
        switch.split_then = 0
        splitcol = Wordindex(com.raw_line,com.splitword)
        Call split_a_line splitcol-1 splitcol
        Iterate com.linecnt
      End

    If switch.split_after_then
      Then Do           /* Maybe split line after THEN/ELSE statement */
        switch.split_after_then = 0
        splitcol = Wordindex(com.raw_line,com.splitword)
        If splitcol > 0
          Then Do                     /* Something following the THEN */
            nextdata = Substr(Strip(Substr(com.raw_line,splitcol)),1,2)
            If nextdata \= com.slashast
              Then Do                    /* Not followed by a comment */
                Call split_a_line splitcol-1 splitcol
                Iterate com.linecnt
              End
          End
      End

    If switch.thenelse                                   /* -> Indent */
      Then Do
        switch.thenelse = 0
        Select
          When com.thenelse_word = Words(p.1)
            Then Do                    /* "then" or "else" on its own */
              switch.reset_indent = 1
              com.next_offset = move_offset(com.offset,pro.indent,1)
            End
          When com.thenelse_word \= 1
            Then Nop                      /* Must be 1st word on line */
          When Word(com.dolist,1) = com.linecnt
            Then Do                         /* "then do" or "else do" */
              com.thendolist = com.linecnt com.thendolist
              switch.incomplete_then = 0
            End
          Otherwise Do
            com.next_offset ,
              = move_offset(com.offset,pro.indent,0-com.thenelse_nest)
            If switch.last_continue
              Then com.next_offset ,
                = move_offset(com.next_offset,pro.cont_indent,-1)
          End
        End                                          /* End of select */
      End
      Else Do           /* Current line does not contain THEN or ELSE */
        If switch.prev_then_target & \switch.continue & com.prev_then = 0
          Then Do
            switch.prev_then_target = 0
            com.last_then_offset = Subword(com.last_then_offset,2)
          End
        If com.prev_then = 0 & switch.incomplete_then
          Then Do                     /* Finished IF THEN IF sequence */
            switch.incomplete_then = 0
            com.offset = com.base_offset
            com.next_offset = com.offset
          End
        com.thenelse_nest = 0
        com.prev_then = 0
      End
    com.thenelse_word = 0

    switch.prev_otherwise = 0
    If switch.otherwise
      Then Do                            /* Maybe indent next line(s) */
        switch.otherwise = 0
        doline = Word(com.dolist,1)
        If Words(p.1) = 1
          Then Do
            switch.prev_otherwise = 1
            com.next_offset = move_offset(com.offset,pro.indent,1)
            com.select_line = Word(com.dolist,1)
          End
      End

    If \switch.continue
      Then switch.last_continue = 0

    Call justify_cmt                    /* Justify comments as needed */

    If out_string \== com.raw_line
      Then Do                             /* Replace line in the file */
        'REPLACE' out_string
        If rc=3
          Then Do                  /* Was new line too long for file? */
            'REPLACE' com.raw_line
            'EXTRACT /ALT'
            'SET ALT' alt.1-2 alt.2-2
          End
      End

    If com.last_char = ',' & \switch.search     /* Continuation line? */
      Then Do
        com.last_char = ' '                   /* Re-set for next time */
        switch.continue = 1
      End
      Else switch.continue = 0

    If com.delim \= ','
      Then Do
        switch.signalonoffv = 0
        com.tokennum = 0
        com.last_key_num = 0
        If \switch.search
          Then switch.skip = 0
      End

    If pro.refresh = 'M' & curll = curltarg
      Then 'REFRESH'
      Else Do
        curll = curll + 1                  /* Modify the current line */
        If curll > curlmax
          Then curll = curlmin

        If pro.refresh = 'M' | refresh_ctr >= pro.refresh
          Then Do                        /* Show our current progress */
            'REFRESH'
            refresh_ctr = 1
          End
          Else refresh_ctr = refresh_ctr + 1
      End
  End com.linecnt                                    /* End main loop */

/*************************** Done processing ***************************/

  ''start_line                   /* Re-position to line we started at */
  'REFRESH'

/******************* Check for Unbalanced delimiters *******************/
  If com.endlist<>''
    Then 'MESSAGEBOX Unbalanced "END"(s) detected at:' com.endlist
  If com.dolist<>''
    Then Do
      temp = ''
      Do i = 1 To Words(com.dolist)
        temp = Word(com.dolist,i) temp
      End
      'MESSAGEBOX Unbalanced "DO"(s) or "SELECT"(s) at:' temp
    End
  If searchkey_list.com.astslash <> ''
    Then Do
      temp = ''
      Do i = 1 To Words(searchkey_list.com.astslash)
        temp = Word(searchkey_list.com.astslash,i) temp
      End
      'MESSAGEBOX Unbalanced comment(s) at:' temp
    End
  If searchkey_list.com.squote<>''
    Then Do
      temp = ''
      Do i = 1 To Words(searchkey_list.com.squote)
        temp = Word(searchkey_list.com.squote,i) temp
      End
      'MESSAGEBOX Unbalanced "''"(s) at:' temp
    End
  If searchkey_list.com.dquote<>''
    Then Do
      temp = ''
      Do i = 1 To Words(searchkey_list.com.dquote)
        temp = Word(searchkey_list.com.dquote,i) temp
      End
      "MESSAGEBOX Unbalanced '""'(s) At:" temp
    End

  Signal exit

/******************* Internal routines and functions *******************/

/**********************************************************************/
/*                                                                    */
/* Scan the current line, looking for special delimiters which force  */
/* us to break up the line into multiple pieces.                      */
/*                                                                    */
/**********************************************************************/
SCAN_LINE:  Procedure Expose p. pindex s switch. com. search_key pro. ,
  searchkey_cnt. searchkey_list. keywords. token
  p. = ''
  pindex = 1                                 /* Tokenized output line */
  com.word_ctr = 0
  switch.donenext = 0
  Do While s \= ''
    If switch.search
      Then Do
        Call find_delim                  /* Look for ending delimiter */
        Iterate
      End
    Parse Var s s1a (com.slashast) s1b +0
    Parse Var s s2a (com.squote) s2b +0
    Parse Var s s3a (com.dquote) s3b +0
    Parse Var s s4a (com.semicolon) s4b +0
    ls1a = Length(s1a)
    ls2a = Length(s2a)
    ls3a = Length(s3a)
    ls4a = Length(s4a)
    minlen = Min(ls1a,ls2a,ls3a)
    If pro.semicolon
      Then minlen = Min(minlen,ls4a)
    Select
      When minlen = Length(s)
        Then Do                                /* No delimiters found */
          p.pindex = p.pindex || scan(s)
          s = ''
        End
      When minlen = ls1a
        Then Do                           /* Found start of a comment */
          switch.search = 1
          search_key = com.astslash
          searchkey_cnt.search_key = searchkey_cnt.search_key + 1
          searchkey_list.search_key ,
            = com.linecnt searchkey_list.search_key
          p.pindex = p.pindex || scan(s1a)
          pindex = pindex + 1
          p.pindex = p.pindex || com.slashast
          s = Substr(s1b,3)
        End
      When minlen = ls3a | minlen = ls2a
        Then Do                            /* Start of literal string */
          If ls3a < ls2a
            Then Do
              search_key = com.dquote
              s2a = s3a
              s2b = s3b
            End
            Else search_key = com.squote
          Parse Var s2b (search_key) s1 (search_key) s +0
          p.pindex = p.pindex || scan(s2a)
          If p.pindex \== ''
            Then addword = (Substr(p.pindex,Length(p.pindex)) == ' ')
            Else addword = 0
          p.pindex = p.pindex || search_key || s1
          If s \= ''
            Then Do                            /* Closing quote found */
              p.pindex = p.pindex || search_key
              com.word_ctr = com.word_ctr ,
                + Words(search_key || s1 || search_key) + addword - 1
              s = Substr(s,2)
              com.delim = search_key
              com.tokennum = com.tokennum + 1
              com.last_token = search_key || s1 || search_key
              search_key = ''
              If Left(s,1) = '('
                Then s1 = function(s1)
              com.last_char = search_key
            End
            Else Do                         /* No closing quote found */
              searchkey_cnt.search_key = searchkey_cnt.search_key + 1
              searchkey_list.search_key = com.linecnt ,
                searchkey_list.search_key
              switch.search = 1
            End
          switch.signalonoffv = 0
        End
      When minlen = ls4a & pro.semicolon
        Then Do                           /* Split line at semi-colon */
          next_chars = Substr(Strip(Substr(s4b,2),'L'),1,2)
          If next_chars = com.slashast
            Then s = Overlay(' ',s,ls4a+1)   /* Don't split this line */
            Else Do
              switch.sc = 1
              com.sc_col = com.num_leading_blanks + ls4a + 1
              Do ctr = 1 To pindex
                com.sc_col = com.sc_col + Length(p.ctr)
              End
              Leave
            End
        End
      Otherwise Do                          /* Just process remainder */
        p.pindex = p.pindex || scan(s)
        s = ''
      End
    End                                              /* End of select */
  End
  If p.pindex = ''
    Then pindex = pindex - 1
  If switch.search & search_key = com.astslash
    Then switch.in_comment = 1
    Else switch.in_comment = 0
  Return

/**********************************************************************/
/*                                                                    */
/* Find the matching delimiter to a single or double quote, or a      */
/* comment.                                                           */
/*                                                                    */
/**********************************************************************/
FIND_DELIM:  Procedure Expose s search_key com. p. pindex ,
  searchkey_cnt. searchkey_list. switch.
  Parse Var s s1 (search_key) s +0
  If search_key = com.astslash
    Then Do
      Do While s \= '' & Right(s1,1)='/'   /* Check for imbedded cmts */
        Parse Var s +1 s
        Parse Var s s2 (search_key) s +0
        s1 = s1'*'s2
      End
      Parse Var s1 . (com.slashast) s2 +0
      Do While s2 \= ''                     /* Found imbedded comment */
        searchkey_cnt.search_key = searchkey_cnt.search_key + 1
        searchkey_list.search_key = com.linecnt searchkey_list.search_key
        Parse Var s2 (com.slashast) (com.slashast) s2 +0
      End
    End
  p.pindex = p.pindex || s1
  If s \= ''
    Then Do                             /* Found the ending delimiter */
      p.pindex = p.pindex || search_key
      s = Substr(s,Length(search_key)+1)
      searchkey_cnt.search_key = searchkey_cnt.search_key - 1
      searchkey_list.search_key = Subword(searchkey_list.search_key,2)
      If searchkey_cnt.search_key <= 0
        Then Do
          If search_key = com.astslash
            Then pindex = pindex+1
            Else Do
              com.delim = search_key
              com.tokennum = com.tokennum + 1
              com.last_token = s1
            End
          switch.search = 0
          search_key = ''
        End
    End
  Return

/**********************************************************************/
/*                                                                    */
/* SCAN - Look for delimiters and break line into tokens.  Delimiters */
/*        such as <> will not be recognized as such, but we are not   */
/*        checking syntax, only looking for delimiters.               */
/*                                                                    */
/**********************************************************************/
SCAN: Procedure Expose switch. com. keywords. pro. p. pindex
  Parse Arg string
  If string = ''
    Then Return string
  delims = ',=()\|&+-;:></%*'
  return_string = ''
  token = ''
  com.delim = ''

  If com.prev_then > 0 & \switch.donenext
    Then Do
      nextword = Translate(Word(string,1))
      Select
        When nextword = 'IF'
          Then Do                /* Incomplete then, save base offset */
            If \switch.incomplete_then
              Then com.base_offset = com.next_offset
            switch.incomplete_then = 1
          End
        When wfind('DO SELECT',nextword) = 0 & com.prev_then = 1
          Then switch.prev_then_target = 1
        Otherwise Nop
      End
    End
    Else switch.donenext = 1

  Select
    When com.word_ctr = 0
      Then last_s1 = ' '
    When p.pindex == ''
      Then last_s1 = ' '
    Otherwise last_s1 = Substr(p.pindex,Length(p.pindex))
  End

  Do i = 1 To Length(string)            /* SAY '****token =' token    */
    s1 = Substr(string,i,1)
    If s1 \= ' ' & last_s1 = ' '
      Then com.word_ctr = com.word_ctr + 1
    last_s1 = s1
    If s1 \= ' '
      Then com.last_char = s1         /* Last character in this token */
    Select
      When Pos(s1,delims) > 0
        Then com.delim = s1                           /* We found one */
      When s1==' '
        Then Do                             /* Treat BLANK separately */
          com.delim = ' '
          If token == ' '
            Then Iterate i                  /* Ignore multiple BLANKs */
        End
      Otherwise Do
        token = token || s1
        Iterate i
      End
    End
    return_string = return_string || scantoken(token) || com.delim
    token = ''
    If com.delim == ';'
      Then Do
        switch.signalonoffv = 0
        com.tokennum = 0
        com.last_key_num = 0
      End
  End i
  save_com_delim = com.delim   /* Save 'com.delim' around 'SCANTOKEN' */
  com.delim = ''
  return_string = return_string || scantoken(token)
  com.delim = save_com_delim                   /* Restore 'com.delim' */
  Return return_string

/**********************************************************************/
/*                                                                    */
/* SCANTOKEN - Classify TOKEN as KEYWORD, VARIABLE, LABEL, or         */
/*             FUNCTION.  Currently, keywords are not scanned exactly */
/*             as the language specifies, i.e. 'FUZZ' is always       */
/*             considered a keyword, not just when it follows         */
/*             'NUMERIC'.  Keywords are checked for last, in case a   */
/*             label or function has the same name as a keyword.      */
/*             Additional tables are provided if you wish to be more  */
/*             exacting.                                              */
/*                                                                    */
/**********************************************************************/
SCANTOKEN: Procedure Expose switch. com. keywords. pro.
  Parse Arg string
  If string = ''
    Then Return string
  return_string = ''

  Do i = 1 To Words(string)
    com.tokennum = com.tokennum + 1
    word1 = Subword(string,i)
    temp = lower(word1)
    Select
      When Datatype(word1) = 'NUM' | Symbol(word1) = 'BAD'
        Then return_string = return_string word1         /* Ignore it */
      When com.delim = ':'                     /* Assume it's a label */
        Then return_string = return_string label(word1)
      When com.delim = '('                  /* Assume it's a function */
        Then return_string = return_string function(word1)
      When (com.last_token=='signal' | com.last_token=='call'),
        & com.tokennum > com.last_key_num
          Then Do
            /* Things after 'SIGNAL' and 'CALL' are assumed to be LABELs*/
            If com.last_token == 'signal' & wfind('on off value',temp) > 0
              Then Do
                switch.signalonoffv = 1
                return_string = return_string keyword(word1)
              End
              Else return_string = return_string call_label(word1)
          End
      When wfind(keywords.scantok,temp) > 0
        Then Do
          If switch.signalonoffv
            Then Do
              switch.signalonoffv = 0
              return_string = return_string label(word1)
            End
            Else return_string = return_string keyword(word1)
        End
      Otherwise Do
        return_string = return_string variable1(word1)
        switch.signalonoffv = 0
      End
    End
    com.last_token = temp
  End
  Return Strip(return_string)

/**********************************************************************/
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
JUSTIFY_CMT:  Procedure Expose p. switch. com. pindex ,
  out_string pro.
  Select
    When p.1 <> ''
      Then Nop
    When p.2 = ''
      Then Nop
    When \pro.cmt_indent | switch.in_comment
      Then Do                         /* Do not adjust comment indent */
        switch.skip = 1
        p.1 = Left('',com.num_leading_blanks)
      End
    When Substr(com.raw_line,1,2) == com.slashast
      Then switch.skip = 1           /* Leave comments in col 1 alone */
    Otherwise Nop
  End

  Select
    When switch.skip
      Then Nop
    When wfind(pro.noindent,Translate(Word(p.1,1))) <= 0
      Then p.1 = Left('',com.offset) || p.1
    When com.offset >= pro.lm
      Then p.1 = Left('',com.offset) || p.1
    Otherwise Nop
  End

  out_string = ''
  Do i = 1 To pindex
    out_string = out_string || p.i
  End

  len1 = Length(out_string)
  If len1 > 1 & pro.cmt_rjust & \switch.search
    Then Do
      If Substr(out_string,len1-1,2) == com.astslash
        Then Do
          /****************** Right Adjust Comment *******************/
          i = Pos(com.slashast,out_string)
          If i > 1
            Then Do                         /* Comment after column 1 */
              comment = Substr(out_string,i)
              line = Strip(Substr(out_string,1,i-1),'T')
              If line <> ''
                Then Do                   /* More than just a comment */
                  i = Length(line)
                  If i + Length(comment) < pro.rm
                    Then Do
                     out_string = line,   /* Adjust comment when room */
                      Right(comment,pro.rm-Length(line)-1)
                    End
                End
            End
        End
    End
  Return

/**********************************************************************/
/*                                                                    */
/* The current word has been identified as a REXX keyword.  We modify */
/* it to have the correct case as requested by the user, then check   */
/* to see if it is a special keyword, requiring special processing.   */
/*                                                                    */
/**********************************************************************/
KEYWORD: Procedure Expose switch. com. pro.
  Parse Upper Arg keyword .

  com.last_key_num = com.tokennum
  word1 = select_case(keyword,pro.keyword_case)
  Select
    When wfind('DO SELECT',keyword) > 0
      Then Do                             /* Start a new indent level */
        com.doindent = com.offset com.doindent
        com.next_offset_list = com.next_offset com.next_offset_list
        com.next_offset = move_offset(com.offset,pro.indent,1)
        com.dolist = com.linecnt com.dolist
      End
    When keyword = 'END'
      Then Do                     /* Restore indent level to previous */
        Parse Var com.doindent com.offset com.doindent
        If pro.endindent
          Then com.offset = move_offset(com.offset,pro.indent,1)
        Parse Var com.next_offset_list ,
          com.next_offset com.next_offset_list
        If com.offset = ''
          Then com.offset = pro.lm - 1
        If com.next_offset = ''
          Then com.next_offset = pro.lm - 1
        If com.dolist = ''
          Then com.endlist = com.endlist com.linecnt
          Else Do                    /* This matches with previous DO */
            Parse Var com.dolist prev_do_line com.dolist
            If prev_do_line = Word(com.thendolist,1)
              Then com.thendolist = Subword(com.thendolist,2)
            If prev_do_line = com.select_line
              Then switch.prev_otherwise = 0
            Do Forever               /* Strip out unused THEN indents */
              Parse Var com.last_then_offset then_line ':' then_offset .
              If then_line <= prev_do_line
                Then Leave
              com.last_then_offset = Subword(com.last_then_offset,2)
            End
            com.last_then_offset ,             /* Restore last offset */
              = then_line':'then_offset com.last_then_offset
          End
        switch.incomplete_then = 0                 /* Closed our THEN */
      End
    When keyword = 'ADDRESS'
      Then switch.upcase = 1                   /* Uppercase next word */
    When keyword = 'WHEN'
      Then switch.prev_when = 1
    When keyword = 'THEN' & pro.split_then & com.word_ctr > 1
      Then Do
        switch.split_then = 1                      /* Split this line */
        com.splitword = com.word_ctr              /* ... at this word */
      End
    When keyword = 'THEN'
      Then Do
        switch.thenelse = 1
        com.thenelse_word = com.word_ctr
        If pro.split_after_then
          Then Do
            switch.split_after_then = 1
            com.splitword = com.word_ctr + 1
          End
        If com.word_ctr = 1
          Then Do                            /* Line starts with THEN */
            com.thenelse_nest = com.thenelse_nest + 1
            com.offset = move_offset(com.prev_offset,pro.indent,1)
          End
        If \switch.prev_when
          Then com.last_then_offset ,
            = com.linecnt':'com.offset com.last_then_offset
        nextword = Translate(Word(com.raw_line,com.word_ctr+1))
        Select
          When com.word_ctr > 1
            Then Nop
          When nextword = '' | Substr(nextword,1,2) = com.slashast
            Then Do                    /* THEN on its own on the line */
              If switch.prev_when
                Then com.prev_then = 2       /* Part of a WHEN clause */
                Else com.prev_then = 1
            End
          When nextword = 'IF'
            Then Do              /* Incomplete THEN, save base offset */
              If \switch.incomplete_then
                Then com.base_offset = com.prev_offset
              switch.incomplete_then = 1
            End
          When wfind('DO SELECT',nextword) = 0 & \switch.prev_when
            Then switch.prev_then_target = 1
          Otherwise Nop
        End
        switch.prev_when = 0
      End
    When keyword = 'ELSE'
      Then Do
        switch.thenelse = 1
        com.thenelse_word = com.word_ctr
        If pro.split_after_then
          Then Do
            switch.split_after_then = 1
            com.splitword = com.word_ctr + 1
          End
        switch.prev_then_target = 0
        Parse Var com.last_then_offset ,
          . ':' com.offset com.last_then_offset
        If com.prev_thenelse = keyword
          Then com.thenelse_nest = com.thenelse_nest + 1
          Else com.thenelse_nest = 1
      End
    When keyword = 'OTHERWISE' & com.word_ctr = 1
      Then switch.otherwise = 1
    Otherwise Nop
  End
  Return word1

VARIABLE1: Procedure Expose pro. switch.
  Parse Arg word1
  If switch.upcase
    Then word1 = Translate(word1)
    Else word1 = select_case(word1,pro.variable_case)
  switch.upcase = 0
  Return word1

LABEL: Procedure Expose pro. com.
  Parse Arg word1
  word1 = select_case(word1,pro.label_case)
  If com.tokennum <= 1
    Then com.offset = 0                     /* Left justify this line */
  Return word1

CALL_LABEL: Procedure Expose pro.
  Parse Arg word1
  word1 = select_case(word1,pro.label_call_case)
  Return word1

FUNCTION: Procedure Expose pro. keywords.
  Parse Upper Arg word1
  If wfind(keywords.func,word1) = 0
    Then word1 = lower(word1)
    Else word1 = select_case(word1,pro.function_case)
  Return word1

SELECT_CASE:  Procedure
  Parse Arg word1, case
  Select
    When case = 'U'
      Then word1 = Translate(word1)
    When case = 'M'
      Then word1 = mixed(word1)
    Otherwise
      word1 = lower(word1)
  End
  Return word1

LOWER: Procedure
  Parse Arg args
  args = Translate(args,,
    'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')
  Return args

MIXED: Procedure
  Parse Arg word1 .
  word1 = lower(word1)
  temp = Substr(word1,1,1)
  temp = Translate( temp  )
  word1 = Overlay(temp,word1,1,1)
  Return word1

/**********************************************************************/
/*                                                                    */
/* Internal function 'GETOPTWORD'                                     */
/* Arguments are -  single token, possibly abbreviated,               */
/*               -  string of valid full-length operands with the     */
/*                  length coded as the first character.              */
/*                                                                    */
/* If first argument is found, the full-length token is returned.  If */
/* not found, return original first argument.                         */
/*                                                                    */
/**********************************************************************/
GETOPTWORD: Procedure
  Parse Arg option, template
  nwords = Words(template)
  Do k=1 To nwords
    optword = Word(template,k)
    len = Left(optword,1)
    optword = Substr(optword,2)
    If Length(option) > Length(optword)
      Then Iterate
    If Abbrev(optword,option,len)
      Then Return optword
  End
  Return option

PARSE_ERROR:
  'MESSAGEBOX Error while parsing arguments:' args
EXIT16:
  ?rc = 16
EXIT:
  Exit ?rc

/**********************************************************************/
/*                                                                    */
/* NoValue and Syntax errors are handled here                         */
/*                                                                    */
/**********************************************************************/
NOVALUE:
  ?sigl=sigl
  ?rc=999
  Signal errexit

SYNTAX:
  ?sigl=sigl
  ?rc=rc

ERREXIT:
  Parse Source os how_called myname .
  'MESSAGEBOX -REXX Error on line' ?sigl 'of' myname':'
  'MESSAGEBOX 'Sourceline(?sigl)

  If ?rc = 999
    Then 'MESSAGEBOX NOVALUE error.' Condition('D')
    Else 'MESSAGEBOX Error' ?rc',' Errortext(?rc)
  Signal exit

/**********************************************************************/
/*                                                                    */
/* Set up default values for all customisable options.  Then check    */
/* for a NAMES file which we will read for profile type options.      */
/* These can be overridden by options from the command line if        */
/* desired. If NOPROF has been specified on the command line, the     */
/* variable NOPROF will have been set to TRUE.                        */
/*                                                                    */
/* The name of the file to be read in is hard-coded in the following  */
/* subprocedure.  If the name of the file is given as a zero length   */
/* character string and this macro is running under OS/2 or Windows,  */
/* the contents of the XPATH environment variable are picked up and   */
/* 'REXXFMT.PRO' appended.  This behaviour may be suppressed by       */
/* using the NOPROF option on the invocaton line or by setting the    */
/* pro.fname variable to '*NOPROF*'.  Setting pro.fname to '*NOPROF*' */
/* always suppresses the profile file, even for Windows and OS/2.     */
/*                                                                    */
/* This procedure also contains the hard-coded option values that are */
/* in effect if the profile file is not found and no options are      */
/* given in the invocation line.  Two formats are supported, the      */
/* original CMS NAMES file format and the more general 'keyword =     */
/* value' for example:                                                */
/*     indent = 2                                                     */
/* Instead of using 0 and 1 for false and true, the character strings */
/* 'false' and 'true' (without the quotes) may be used, e.g.          */
/*     cmt_indent = true                                              */
/*     split_them = false                                             */
/*                                                                    */
/**********************************************************************/
READ_PROFILE:  Procedure Expose pro. os. filespec_os noprof
/*pro.fname  = 'D:\X2\REXXFMT.PRO' */         /* Name of profile file */
/*pro.fname  = '*NOPROF*'          */      /* never us a profile file */
  pro.fname  = ''              /* get XPATH and append 'REXXFMT.PRO ' */
  pro.indent = 2                        /* Number of spaces to indent */
  pro.cont_indent = 2     /* Number of spaces to indent continuations */
  pro.variable_case = 'L'                     /* Change variable case */
  pro.label_case = 'U'                           /* Change label case */
  pro.label_call_case = 'L'                 /* Change label call case */
  pro.function_case = 'M'                     /* Change function case */
  pro.keyword_case = 'M'                       /* Change keyword case */
  pro.cmt_indent = 1          /* 1 = indent comments, 0 = leave alone */
  pro.cmt_rjust = 1    /* 1 = right justify comments, 0 = leave alone */
  pro.lm = 3                                  /* Left margin (INDENT) */
  pro.split_then = 1                   /* Split if statements at THEN */
  pro.split_after_then = 0          /* Split lines after THEN or ELSE */
  pro.semicolon = 1                     /* Split lines at semi-colons */
  pro.refresh = 1                      /* Refresh screen after 1 line */
  pro.endindent = 0                   /* Do not indent END statements */
  pro.indent_threshold = 20           /* Threshold for 1 byte indents */
  pro.rm = 72                 /* Right margin for justifying comments */
  pro.noindent = ''                   /* Keywords to force flush left */

/*  set up info for validity checking the parameters read in          */

  valid.            = 'Unknown'

/*  cont_indent     - contains the number of spaces to indent lines   */
/*                    that are continued (i.e. end in a comma)        */
  valid.cont_indent = 'NUM'

/*  indent          - the number of spaces to indent new loops etc.   */
  valid.indent      = 'NUM'

/*  variable_case   - the case for all variables.  Options are U, M,  */
/*                    and L for UPPERCASE, Mixed case, and lower case */
/*                    respectively.                                   */
  valid.variable_case = 'U M L'

/*  label_case      - the case for all labels.  Options are U, M, and */
/*                    L and have the same meanings as above.          */
  valid.label_case    = 'U M L'

/*  label_call_case - the case for all calls to labels.  Options are  */
/*                    U, M, and L and have the same meanings as above.*/
  valid.label_call_case='U M L'

/*  keyword_case    - the case for all REXX keywords.  Options are U, */
/*                    M, and L and have the same meanings as above.   */
  valid.keyword_case  = 'U M L'

/*  function_case   - the case for all internal REXX functions.       */
/*                    Options are U, M, and L and have the same       */
/*                    meanings asabove.                               */
  valid.function_case = 'U M L'

/*  lm              - the left margin for starting indentation        */
  valid.lm          = 'NUM'

/*  rm              - the right margin for the end of comments        */
  valid.rm          = 'NUM'

/*  cmt_rjust       - A flag telling whether or not to try to right   */
/*                    adjust comments to the right margin.  The       */
/*                    default is 1 (yes).                             */
  valid.cmt_rjust   = '1 0'

/*  cmt_indent      - A flag specifying whether or not to indent      */
/*                    single line comments at the current margin.     */
/*                    The default is 1 (yes)                          */
  valid.cmt_indent  = '1 0'

/*  split_then      - A flag specifying whether or not to split an IF  */
/*                    statement at the THEN statement.  If 1 (on - the */
/*                    default), all THEN statements will be indented   */
/*                    to begin on a new line.  If 0, lines will not be */
/*                    split at THEN statements.                        */
  valid.split_then  = '1 0'

/*  split_after_then - A flag specifying whether or not to split a     */
/*                    line so that the clause following a THEN or ELSE */
/*                    statement should be formatted to a new line.     */
/*                    The default value, 0, means leave the line as    */
/*                    is.  If set to 1, this option will split lines   */
/*                    after THEN statements.  Note that this option in */
/*                    conjunction with the SPLIT_THEN option will      */
/*                    force THEN and ELSE statements to occupy a       */
/*                    single line.                                     */
  valid.split_after_then   = '1 0'

/*  semicolon       - Controls whether to split lines containing       */
/*                    semi-colons (;) into separate lines.  Default is */
/*                    1, which means to split the lines                */
  valid.semicolon   = '1 0'

/*  noindent        - A list of keywords to be forced flush left if    */
/*                    found at the beginning of the line.              */
  valid.noindent    = 'ANY'

/*  refresh         - Not used. Accepted but ignored for compatibility */
/*                    with CMS.                                        */
  valid.refresh     = 'ANY'

/*  endindent       - A flag indicating whether or not to indent END   */
/*                    statements so they line up with the code, or are */
/*                    indented with the rest of the DO/SELECT block.   */
/*                    The default value, 0, means do not indent END    */
/*                    statements, i.e. line them up with the line that */
/*                    contains the preceding DO.                       */
  valid.endindent   = '1 0'

/*  indent_threshold - A number indicating a column, after which the   */
/*                    INDENT and CONT_INDENT variables automatically   */
/*                    revert to 1. The purpose of this option is to    */
/*                    allow heavily nested code a better chance of     */
/*                    remaining on a single page.  The default value   */
/*                    is 20.                                           */
  valid.indent_threshold  = 'NUM'

  opencomm  = '/'||'*'
  fname = pro.fname
  If (noprof = 1)
    Then Do
      fname = '*NOPROF*'                      /* name of profile file */
    End                                                     /* end if */

  If (fname = '')
    Then Do                                 /* profile name not given */
      If (os.1 = 'OS/2')
        Then Do                                               /* os/2 */
          fname =  Value('XPATH',,'OS2ENVIRONMENT') || '\REXXFMT.PRO'
        End                                                 /* end if */
      If (Translate(Left(os.1,7)) = 'WINDOWS')
        Then Do                                               /* os/2 */
          fname =  Value('XPATH',,'ENVIRONMENT') || '\REXXFMT.PRO'
        End                                                 /* end if */
    End                                                     /* end if */

  'EXTRACT /ESCAPE/'
  If (fname <> '') & (fname <> '*NOPROF*')
    Then Do                                      /* file name given ? */
      /* 'QUERY EXISTS' returns full path or null string  if not o.k. */
      state = Stream(fname,'C','QUERY EXISTS')
      If (state =  '')
        Then Do
          'MESSAGEBOX Cannot find profile file' fname '!'escape.1'nHit',
            '"A" to abort, anything else continues'escape.1'nwith' ,
            'hard-coded options'
          key_entered = result
          If (Translate(key_entered) = 'A')
            Then Do
              Signal exit16
            End                                             /* end if */
        End
        Else Do
          'MESSAGEBOX using profile file' fname','escape.1'nHit',
            '"A" to abort, anything else continues'
          key_entered = result
          If (Translate(key_entered) = 'A')
            Then Do
              Signal exit16
            End                                             /* end if */
          Call stream fname,'C','OPEN READ'
          Do While (Lines(fname) <> 0)
            profile_line = Translate(Strip(Linein(fname)))

            If (profile_line = '') ,                    /* empty line */
              |  (Left(profile_line,1) = '*') ,     /* it's a comment */
              |  (Left(profile_line,2) = opencomm)
                Then Do                             /* it's a comment */
                  Iterate
                End                                         /* end if */

            If (Left(profile_line,6) = ':NICK.')
              Then Do
                Iterate
              End                                           /* end if */

            profile_line = Strip(profile_line,'Leading',':') /* remove*/
            profile_line = Translate(profile_line,'  ', '.=')/* : = . */
            Parse Var profile_line variable data
            data     = Strip(data)                   /* remove blanks */

            If (data = 'TRUE') | (data = 'YES')
              Then Do
                data = 1
              End                                           /* end if */
            If (data = 'FALSE') | (data = 'YES')
              Then Do
                data = 0
              End                                           /* end if */

            /*  validity check the variable against the data          */

            If (valid.variable = valid.)
              Then Do                          /* should not occur,.. */
                'MESSAGEBOX Unknown option:' ,/* .see CHECK_ARGSTRING */
                  variable', ignored.'
              End                                           /* end if */

            If (valid.variable = 'NUM')
              Then Do
                If (Datatype(data) <> 'NUM')
                  Then Do
                    'MESSAGEBOX Data for' variable ,
                     'is not numeric' data', ignored.'
                  End                                       /* end if */
              End                                           /* end if */

            If (valid.variable = 'U M L')
              Then Do
                If (Wordpos(data,valid.variable) = 0)
                  Then Do
                    'MESSAGEBOX data for' variable ,
                     'not u, m or L:' data'ignored.'
                  End                                       /* end if */
              End                                           /* end if */

            If (valid.variable = '1 0')
              Then Do
                If (Wordpos(data,valid.variable) = 0)
                  Then Do
                    'MESSAGEBOX data for' variable ,
                     'not 1 or 0 (true or false):' data',ignored.'
                  End                                       /* end if */
              End                                           /* end if */

            pro.variable = data
          End
          Call stream fname,'C','CLOSE'
        End
    End                                     /* end if file name given */
    Else Do
      'MESSAGEBOX not using a profile file'escape.1'nHit',
        'any key to continue'
    End                                                   /* end else */
  Return
/**********************************************************************/
/*                                                                    */
/* Check the argument string for any options                          */
/*                                                                    */
/**********************************************************************/
CHECK_ARGSTRING:  Procedure Expose first_line ?rc pro.
  Arg opts
  valid_opts = '1INDENT 1LM 1LEFTMARGIN 1RM 1RIGHTMARGIN'
  Do While opts <> ''
    Parse Var opts . opts 1 optword temp2 .
    optword = getoptword(optword,valid_opts)
    Select
      When optword = 'INDENT'
        Then Parse Var opts pro.indent opts
      When optword = 'LEFTMARGIN' | optword = 'LM'
        Then Parse Var opts pro.lm opts
      When optword = 'RIGHTMARGIN' | optword = 'RM'
        Then Parse Var opts pro.rm opts
      Otherwise Do                                  /* Invalid option */
        'MESSAGEBOX Invalid option '''optword'''.'
        Signal exit16                                        /* Exit. */
      End
    End
  End

/**********************************************************************/
/*                                                                    */
/* Check the options for validity.                                    */
/*                                                                    */
/**********************************************************************/
  Select
    When \Datatype(pro.indent,'N')
      Then Do
        'MESSAGEBOX INDENT operand '''pro.indent'''',
          'is not numeric.'
        Signal exit16                             /* Take error exit. */
      End
    When \Datatype(pro.lm,'N')
      Then Do
        'MESSAGEBOX LEFTMARGIN operand '''pro.lm'''',
          'is not numeric.'
        Signal exit16                             /* Take error exit. */
      End
    When pro.lm < 0
      Then Do
        'MESSAGEBOX LEFTMARGIN operand '''pro.lm'''',
          'is not numeric.'
        Signal exit16                             /* Take error exit. */
      End
    When \Datatype(pro.rm,'N')
      Then Do
        'MESSAGEBOX RIGHTMARGIN operand '''pro.rm'''',
          'is not numeric.'
        Signal exit16                             /* Take error exit. */
      End
    Otherwise Nop                                   /* Options are ok */
  End

  If pro.refresh = 0
    Then pro.refresh = 99999999
  Return

/**********************************************************************/
/*                                                                    */
/* Split a line into two pieces.  We must adjust our do loop          */
/* parameters accordingly.  Also, since we will be re-scanning the    */
/* current line, we will adjust our indent counters if they were      */
/* modified by this line.                                             */
/*                                                                    */
/**********************************************************************/
SPLIT_A_LINE:  Procedure Expose com. switch. last_line
  Parse Arg last_col first_col .
  current_line = Substr(com.raw_line,1,last_col)
  next_line = Substr(com.raw_line,first_col)
  line_added = 0

  'CURSOR' com.linecnt first_col
  'SPLIT'
  line_added = 1

  If current_line = ''
    Then 'REPLACE  '
    Else 'REPLACE' current_line
  If line_added
    Then last_line = last_line + 1

  If Word(com.dolist,1) = com.linecnt
    Then Do                          /* Undo our do indent processing */
      com.doindent = Subword(com.doindent,2)
      com.next_offset_list = Subword(com.next_offset_list,2)
      com.dolist = Subword(com.dolist,2)
    End

  lastword = Translate(Word(current_line,Words(current_line)))
  If lastword = 'THEN'
    Then Do                                   /* Undo THEN processing */
      If Words(current_line) = 1
        Then                                 /* Line starts with THEN */
          com.thenelse_nest = com.thenelse_nest - 1
      com.last_then_offset = Subword(com.last_then_offset,2)
    End
  If lastword = 'ELSE'
    Then Do                                   /* Undo ELSE processing */
      If com.prev_thenelse = 'ELSE'
        Then com.thenelse_nest = com.thenelse_nest - 1
      com.last_then_offset ,
        = com.linecnt':'com.offset com.last_then_offset
    End

  Parse Var com.save_offset com.prev_offset com.offset com.next_offset
  switch.split = 1
  com.linecnt = com.linecnt - 1
  Return

/**********************************************************************/
/*                                                                    */
/* Move an offset by the specified amount.  We check to see if we're  */
/* past the indent threshold before applying the delta, since we only */
/* want to indent one space past the threshold.                       */
/*                                                                    */
/**********************************************************************/
MOVE_OFFSET:  Procedure Expose pro.
  Parse Arg startoffset, delta, num_deltas
  newoffset = startoffset + num_deltas*delta
  Select
    When startoffset <= pro.indent_threshold ,
      & newoffset <= pro.indent_threshold
        Then Nop                                /* All below threshold*/
    When startoffset > pro.indent_threshold ,
      & newoffset > pro.indent_threshold
        Then newoffset = startoffset + num_deltas/* Just use 1 indent */
    When num_deltas > 0
      Then Do                          /* Crossing line left to right */
        newoffset = startoffset
        Do num_deltas
          If newoffset < pro.indent_threshold
            Then newoffset = newoffset + delta
            Else newoffset = newoffset + 1
        End
      End
    Otherwise Do                       /* Crossing line right to left */
      newoffset = startoffset
      Do Abs(num_deltas)
        If newoffset > pro.indent_threshold
          Then newoffset = newoffset - 1
          Else newoffset = newoffset - delta
      End
    End
  End
  Return newoffset

/*********************** replacement for "find" ***********************/
/*   note: this assmues that find()  was used to detect words, this   */
/*      lead to errors, e.g. with  find('DO SELECT,var) when the      */
/*  contents of var are a substing of 'DO SELECT', for example 'S'.   */
/*     As a result, the name has been changed to reflect wordfind     */
/*********************** replacement for "find" ***********************/

WFIND:
  If (Arg(3,'E'))
    Then Do
      Return Wordpos( Arg(2), Arg(1), Arg(3) )
    End                                                     /* end if */
    Else Do
      Return Wordpos( Arg(2), Arg(1) )
    End                                                   /* end else */

/*======================= End of Macro REXXFMT =======================*/
