/*****************************************************************************/
/* RF.X:  Macro to ReFlow a paragraph of text in an intelligent manner.      */
/*                                                                           */
/* Author: Craig Raymond (CRAYMOND at GDLVM6)                                */
/*****************************************************************************/
/* Syntax: RF <left_margin> <right_margin> <paragraph_indent>                */
/*                                                                           */
/* RF will reflow a paragraph of text to the margins supplied, then move the */
/*    cursor to the next paragraph (next non-blank line).  If no margins are */
/*    supplied, RF will use the default margins for the filetype being       */
/*    edited.  RF will use the first 'comment margin' for the paragraph      */
/*    indentation. The cursor position is used to determine to starting line */
/*    for the paragraph to be reflowed.                                      */
/*****************************************************************************/
/* This macro is dedicated to Bob Rankin.  Bob wrote the original macro      */
/* which works in XEDIT and in CPEDIT (DOS and OS/2 flavors). Thanks Bob!    */
/*****************************************************************************/
/* Revision history:                                                         */
/* January   08, 1994 - Original release.                              (1.00)*/
/* January   10, 1994 - Added ability to pull margins and indentation  (1.01)*/
/*                      from X.                                              */
/*****************************************************************************/

parse arg lmar rmar indent

/*****************************************************************************/
/* Use input values if any, else use the defaults for the filetype in use.   */
/*****************************************************************************/

'EXTRACT /MARGINS/'
dfltlmar = MARGINS.1                             /* Default left margin      */
dfltrmar = MARGINS.2                             /* Default right margin     */
dfltindent = MARGINS.3                           /* Default indentation      */

If lmar <> '' Then Do                            /* Check left margin value  */
   If (datatype(lmar,'W')) = 0 Then Do
      MSG 'Left margin must be a whole number.'
      Exit 99
   End
End
Else lmar = dfltlmar

If rmar <> '' Then Do                            /* Check right margin value */
   If (datatype(rmar,'W')) = 0 Then Do
      MSG 'Right margin must be a whole number.'
      Exit 99
   End
End
Else rmar = dfltrmar

If indent <> '' Then Do                          /* Check indentation value  */
   If (datatype(indent,'W')) = 0 Then Do
      MSG 'Indentation must be a whole number.'
      Exit 99
   End
End
Else indent = dfltindent

/*****************************************************************************/
/* Snarf all the words (in the paragraph) into an array (list of words).     */
/*****************************************************************************/

i=0                                              /* Original line counter    */

do forever
   'EXTRACT /CURLINE/'                           /* End of paragraph is      */
   if curline.1 = '' then leave                  /* blank line,              */
   if substr(curline.1,1,1) = ':' then leave     /* colon, or                */
   if substr(curline.1,1,1) = '.' then leave     /* period in column one     */
   do words(curline.1)
      i = i + 1
      parse var curline.1 word.i curline.1       /* Extract 1st word fm line */
   end
   'DELETE'                                      /* Line complete, delete it */
   if rc<>0 then leave
end

if i>0 then 'UP'                                 /* Move up one line         */

/*****************************************************************************/
/* Recreate the paragraph with the new margins.                              */
/*****************************************************************************/

added = 0                                        /* Counter for new lines    */
/* newline = copies(' ',lmar-1+indent) */        /* Start a new line         */
newline = copies(' ',indent-1)                   /* Start a new line         */
do j = 1 to i                                    /* Loop for all words       */

   /**************************************************************************/
   /* If length of next word would cause line to pass 'rmar', add the line   */
   /* (without new word) then start a new line, else continue building       */
   /* current line.                                                          */
   /**************************************************************************/

   if length(newline)+length(word.j) > rmar then Call Addline
   newline = newline word.j                      /* Build new line           */

   /**************************************************************************/
   /* If last character is "." or "?" or "!", then add extra space.  There   */
   /* should be two spaces between the end and beginning of sentences, and   */
   /* since 'words' strips extra spaces, we need to add one.                 */
   /**************************************************************************/

   lastchar = substr(reverse(word.j),1,1)        /* Get last char of word    */
   if lastchar = '.' | lastchar = '?' | lastchar = '!' then   /* If end punc */
      newline = newline||' '                     /* Add space                */
end
if newline <> '' then Call Addline               /* Put last line in buffer  */

/*****************************************************************************/
/* Reposition to top of next paragraph (maybe).                              */
/*****************************************************************************/
/* Put the cursor below the currently reflowed paragraph, and move down      */
/* until we find a non-blank line, if we find EOF then EXIT.                 */
/*****************************************************************************/

'DOWN'                                           /* Get under reflwed prgrph */
'EXTRACT /CURLINE'                               /* Get current line         */

Do until curline.1 <> ''
   'DOWN'                                        /* move down one line       */
   If rc <> 0 Then Exit                          /* We hit EOF, we're done   */
   'EXTRACT /CURLINE/'                           /* Get current line         */
End

Exit

/*****************************************************************************/
/* Add a new line to the file.                                               */
/*****************************************************************************/
Addline:
  added = added + 1                              /* Incr newline counter     */
  newline = substr(newline,2)                    /* Remove leading space     */
  'INPUT' newline                                /* Put line in file         */
  newline = copies(' ',lmar-1)                   /* Start a new line         */
Return
