/******************************** SHOWNEST *********************************/
/*                        X2 Editor Macro SHOWNEST                         */
/*            SHOW the NESTing of ( ) brackets in an expression            */
/*                                                                         */
/* This macro scans the line the cursor is positioned in and tries to      */
/* analyse the 'nesting' of the brackets within that line.  Comments and   */
/* multiple blanks are removed (comments must, however, be complete on one */
/* line).  Continuation commas (we assume we are editing REXX) are         */
/* honoured (the next line is also processed) but the comma itself is      */
/* removed.  If the language is C, all lines are processed until either an */
/* empty line is found or a line is found that contains either a ";" or    */
/* "{" (curly bracket, '7B'X on PC) character.                             */
/*                                                                         */
/* Any other language just processes the one line as we do                 */
/* not know how continuations are indicated.  The language (REXX, C or     */
/* none) is determined by 'EXTRACT code_type'.                             */
/*                                                                         */
/* The results are displayed in a popup window.  However, this behaviour   */
/* is customisable:  the results may be written to a file in the ring      */
/* called SHOWNEST.$$$ (refer to the 'use' below).                         */
/*                                                                         */
/* This version of this macro ignores bracket characters in quoted         */
/* strings.  'Brackets' means only round brackets, i.e. "(" or ")".        */
/* Others, e.g. curly brackets, arrowheads etc., are NOT considered.       */
/* Comments are removed.  Note that this macro cannot handle nested        */
/* comments properly.  Version 1.96 (or higher) is required due to the use */
/* of MESSAGEBOX.  This is checked for in the macro.                       */
/*                                                                         */
/* This macro is known to work under OS/2 and WindowsNT. The author        */
/* believes (but cannot guarantee) that the only system dependencies are   */
/* 1) the 'filespec' bif for picking up the name of this macro; this name  */
/*    is only used for displaying (error messages etc),                    */
/* 2) the 'beep' bif wich is only used with error messages (customisable,  */
/*    see variable 'sound_beep' below),                                    */
/* 3) the hexadecimal values for the box characters (these may be          */
/*    replaced by any characters chosen by users) and curly brackets.      */
/* Issue 'ALL /filespec(/',  'ALL/"X/' and 'ALL /SOUND_BEEP/' to see the   */
/* relevant lines).                                                        */
/*                                                                         */
/* Call:                                                                   */
/*                                                                         */
/*      SHOWNEST ?                                                         */
/*                                                                         */
/*      puts out these comments as a help text                             */
/*                                                                         */
/*      SHOWNEST                                                           */
/*                                                                         */
/*      i.e. without parameters, is the normal invocation.  This leads to  */
/*      the current line (the line the cursor is in) being scanned.  The   */
/*      nesting of the brackets in that line, which is assumed to contain  */
/*      an expression, is displayed.  This is useful if REXX has           */
/*      complained about that line, e.g. with SYNTAX or some such.         */
/*                                                                         */
/* Author:  Roger C. Spencer  (V2SPENCE at SDFVM2)                         */
/*                                                                         */
/* Change activity                                                         */
/* 99-11-03 RCS New                                                        */
/* 00-03-08 RCS Continuation not recognised before a comment in same line  */
/***************************************************************************/

 Signal endhelpline
 endhelpline: lasthelpline = sigl -1

/*-------------------------------------------------------------------------*/
/* N O T E !  keep the above two lines as they are used to pick up         */
/* the number of the last line of help text to put out to a window.        */
/*-------------------------------------------------------------------------*/

/* Customization section                                                   */
/* ------------------------------------------------------------------------*/

sound_beep= 'YES'                /* 'YES' = sound beeper in case of errors.*/
                                            /* set to 'NO' if not required */

min_version = 1.96     /* X version number that first supported MESSAGEBOX */

use = 'WINDOW'   /* 'FILE' to always write the results to a (new) file.    */
                 /* 'WINDOW' to always write the results to a popup window.*/
                 /* 'CHOSE' to write the results to a popup window if the  */
                 /* results are expressable in lines of max. 80 bytes or,  */
                 /* if they are not, then write to a new file called       */
                 /* SHOWNEST.$$$ (this option dates from the time when     */
                 /* popup windows were not left/right scrollable).  This   */
                 /* macro does not check to see if a file called           */
                 /* SHOWNEST.$$$ already exists.                           */

                 /* Setting 'use' to any value other than one of the  ..   */
                 /* .. above 3 may lead to unpredicable results.           */

use = Translate(use)                           /* make sure it's uppercase */

rexxct = 'REXX'                                      /* CODE_TYPE for REXX */
cct    = 'C'                                              /* CODE_TYPE "C" */

/* Get name of this macro (for use in error messages etc.)                 */
/* ------------------------------------------------------------------------*/

Parse Source . . fullpath .
myname = Translate(filespec('Name', fullpath))
Parse Var myname myname  "." .         /* remove the extension             */

/* get name of file being edited                                           */
/* ------------------------------------------------------------------------*/

'EXTRACT /RING/'
edited_file = ring.1

/* Pick up option, see if help requested                                   */
/* ------------------------------------------------------------------------*/

Arg parm                /* assume opts to be passed to editor at invocation*/

'EXTRACT /CURSOR/'                     /* Get current position             */
save_row = cursor.1
save_col = cursor.0

'EXTRACT /FLSCREEN/'                   /* get no. of lines on screen       */
If (flscreen.0 <> 2) Then window_size = 18     /* set up no. of lines...   */
Else Do                                /* ...for pop-up windows            */
   window_size = (flscreen.2 - flscreen.1 + 1)- 4
End

help = '? /? -? /H -H HELP'
If (wordpos(parm, help) <>0) Then Do
   window_width = Length(sourceline(1))
   'WINDOW' window_size window_width lasthelpline ,
            myname 'Help text'                         /* define window    */
   Do i = 1 To lasthelpline-1          /* make sure loop ends sometime     */
      'WINLINE' sourceline(i) || '\nREFRESH'
   End
   Exit 0
End

/* Check X2 version number. If not high enough, abort.                     */
/* ---------------------------------------------------                     */

'EXTRACT /VERSION/'

If (rc <> 0) Then Do          /* set to minimal version number if not o.k. */
   Version.1 = 0.0
End
Else Do
   Version.1 = Left(version.1,4)                     /* drop "Beta a" etc. */
End                                                            /* end else */
If (version.1 < min_version) Then Do            /* MESSAGEBOX  available ? */
   Call beepit                                 /* give up if not available */
   'MSG Version too low:' ,
     myname 'needs version' min_version', your editor has version' Version.1
   Exit
End

/* ------------------------------------------------------------------------*/
/* Decide which programming language. This is used to define how lines     */
/* are continued (onto the next line). Supported types are "C", "REXX"     */
/* and "NONE"; these values are stored in the variable "language".         */
/* REXX = remove comments and trailing blanks. If the last character in    */
/*        the line is ",", then assume the next line is a continunation.   */
/* C    = keep on adding lines until a line is found containing a ";"      */
/*        (normal end of an expression) or "{" (closes an IF etc.) or      */
/*        until an empty line is found. If the last character in the line  */
/*        is a backslash "\", assume it's a continuation character (used   */
/*        in macro definitions) process the next line but remove the "\".  */
/* NONE = Ah well, just look at current line, i.e. no continuations        */
/*                                                                         */
/* The above assumes that the expression to be checked starts at the       */
/* beginning of the line and is either continued on to the next line or    */
/* teminates on the current line.                                          */
/*                                                                         */
/* It is realized that the rules above are vastly oversimplified (in C the */
/* ; may be in a quoted string, for example) but they correspond, more     */
/* or less, to the author's programming style.                             */
/* ------------------------------------------------------------------------*/

'EXTRACT /CODE_TYPE/'                              /* get current language */

Select

  When (wordpos(CODE_TYPE.1, rexxct) <> 0) Then Do
     language = "REXX"
  End

  When (wordpos(CODE_TYPE.1, cct) <> 0) Then Do
     language = "C"
  End

  Otherwise Do
     language = "NONE"
  End

End

'MESSAGEBOX' myname': Assumed programming language =' language

/* pick up current line, check if it is continued, get continuation lines  */
/* ----------------------------------------------------------------------  */

com_beg = "/"||"*"                               /* begin of comment chars */
com_end = "*"||"/"                                 /* end of comment chars */
bs = "\"                                                 /* bs = backslash */

scanme = ""                   /* scanme = just contains bracket chars "()" */
linex  = ""      /* linex = space(source line(s))/*without comments etc.*/ */

'EXTRACT /SIZE/'                                /* number of lines in file */

Do i = save_row To size.1 By 1                  /* max. scan = end of file */

   'EXTRACT /LINE' i'/'                                /* get current line */

   line = space(line.1)
   Do While (pos(com_beg,line) <> 0)                    /* Remove comments */
      Parse Var line line1 (com_beg) line2 (com_end) line3
      line = line1 || line3
   End                                                            /* End do*/

   If (linex = "") Then Do
      linex  = space(line)
   End
   Else Do
      linex  = linex space(line)
   End                                                         /* end else */

   If (language = "NONE") Then Do
      Leave              /* quit loop, we know nothing about continuations */
   End

   If (language = "REXX") Then Do
      If (right(linex,1) <> ",") Then Do
         Leave                  /* quit loop if not continuation character */
      End
      linex = delstr(linex, Length(linex)) /* drop cont. char for analysis */
   End

   If (language = "C") Then Do
      If (line = "") Then Do
         Leave                                  /* quit loop if empty line */
      End                                                        /* end if */
      If (right(linex,1) = "\") Then Do       /* line is continued on next */
         linex = delstr(linex, Length(linex))           /* drop cont. char */
         Iterate
      End                                                        /* end if */
      /* the hex string in the second pos-bif is an opening curly bracket  */
      If ( (pos(";", line) <> 0) | (pos("7B"X, line) <> 0)) Then Do
         Leave                            /* quit loop if end of statement */
      End
   End

End                                                               /* End do*/

/* scan through line,  check for bracket chars in quoted strings           */
/* -------------------------------------------------------------           */

sq = "'"                                                   /* Single Quote */
dq = '"'                                                   /* Double Quote */
oc = '?'                   /* character to Overlay bracket Chars in string */
scanme = linex                                        /* copy line to scan */

If ((pos(sq,scanme)<>0) | (pos(dq,scanme)<>0)) Then Do
   status = 'NORMSCAN'          /* normal scanning, ie. outside of strings */
   Do i = 1 To Length(scanme)
      c = Substr(scanme,i,1)                           /* get current char */
      Select

         When (status='NORMSCAN') Then Do
            Select
               When (c = sq) Then Do
                  status = 'SQS'                   /* single quoted string */
               End                                             /* end when */
               When (c = dq) Then Do
                  status = 'DQS'                   /* double quoted string */
               End                                             /* end when */
               Otherwise Do
                  Nop
               End                                        /* end otherwise */
            End                                              /* end select */
         End                                                   /* end when */

         When ((status='SQS') | (status='DQS')) Then Do       /* in string */
            Select
               When ((status='SQS') & (c = sq)) Then Do
                  status = 'NORMSCAN'       /* end of single quoted string */
               End
               When ((status='DQS') & (c = dq)) Then Do
                  status = 'NORMSCAN'       /* end of double quoted string */
               End
               When (C = "(") | (C = ")") Then Do       /* remove bracket..*/
                  scanme = Overlay(oc, scanme, i)           /* from string */
               End
               Otherwise Do
                  Nop
               End                                        /* end otherwise */
            End                                              /* end select */
         End                                                   /* end when */

         Otherwise Do
            Nop
         End                                              /* end otherwise */

      End                                                    /* end select */
   End                                                           /* end do */
End                                                              /* end if */

/*  Blank everything in 'scanme' except the bracket characters             */
/*  ----------------------------------------------------------             */

scanme = Translate(scanme, "()", "()" || xrange("00"X, "FF"X))

If (scanme = "") Then Do
   'MESSAGEBOX' myname': no brackets found'
   Exit 0
End                                                              /* end if */

/* scan through line, pick out bracket characters                          */
/* ----------------------------------------------                          */

max_nest = 0
cur_nest = 0
shadow = ""                            /* shadow line contains nesting ... */
nl_indic = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'/* ... level indicator    */
cols.  = ""
Do i = 1 To Length(scanme)

   cur_char = Substr(scanme, i, 1)

   Select
     When cur_char = "(" Then Do
        cur_nest = cur_nest + 1
        max_nest = max(max_nest, cur_nest)
        shadow   = shadow || Substr(nl_indic,cur_nest+1,1,'*')      /*     */
        cols.cur_nest = cols.cur_nest i              /* save char position */
     End
     When cur_char = ")" Then Do
        shadow   = shadow || Substr(nl_indic,cur_nest+1,1,'*')      /*     */
        If (cur_nest=0) Then Do
           Call beepit
           'MESSAGEBOX' myname': too many ")" chars'
           Leave
        End
        Else Do
           cols.cur_nest = cols.cur_nest i           /* save char position */
           cur_nest = cur_nest - 1
        End                                                    /* end else */
     End
     Otherwise Do
        shadow   = shadow || " "
     End
   End

End                                                               /* End do*/
If (cur_nest <>0) Then Do
   Call beepit
   'MESSAGEBOX' myname': unmatched "(" ")" chars'
End

Do i = 1 To max_nest
   If ((words(cols.i) // 2) <> 0) then do       /* if odd number of col... */
      cols.i = cols.i Length(scanme)+1            /* .. numbers, add dummy */
   End
End                                                               /* End do*/

/* Put out to popup window or, if line is too long, a new file in the ring */
/* ----------------------------------------------------------------------- */

lines_written = 0              /* number of lines written to file / window */
position_cursor = 1           /* row number if ":" not passed to 'writeme' */
If ( (use = 'FILE') | ,
    ((length(linex)+2 > 80) & (use = 'CHOSE') ) ,
   ) Then Do
   use = 'FILE'                            /* Show we're writing to a file */
   tempfile = myname || '.$$$'                          /* open a temp file*/
   'X' tempfile                                    /* new file in the ring */
   'CURSOR 1 1'                         /* position to top of the new file */
   'MARGINS 1' Length(linex)+2 '1 100'    /* make sure line is long enough */
   'X' edited_file                          /* return to file being edited */
End
Else Do
   use = 'WINDOW'                       /* Show we're using a popup window */
   'WINDOW 15' Length(linex)+2 5+max_nest myname 'Brackets'
   If (rc<>0) Then Do
      Call beepit
      'MESSAGEBOX' myname': rc' rc 'from WINDOW, exiting'
      Exit rc
   End
   Call writeme "   "                /* blank line for popup window cursor */
End                                                            /* end else */

/* loop to put out the 'gantries' connecting the (paired) brackets         */
/* ---------------------------------------------------------------         */

Do i = 1 To max_nest                      /* 'outermost' brackets have i=1 */

   temp = ""

   Do j = 1 To i

      Do k = 1 To words(cols.j)
         If (j<>i) Then Do                                 /* Vertical bar */
            temp = Overlay("B3"X, temp, word(cols.j, k) )
         End
         Else Do
            temp = Overlay("DA"X, temp, word(cols.j, k) )  /* left+right.. */
            temp = Overlay("BF"X, temp, word(cols.j, k+1) )   /* ..corners */
            junk = copies( "C4"x, (word(cols.j,k+1)-word(cols.j,k))-1)
            If (junk<>"") Then Do        /* zero or more horizontal dashes */
               temp = Overlay(junk, temp, (word(cols.j, k)+1) )
            End
            k = k+1                                    /* process in pairs */
         End                                                   /* end else */
      End                                                         /* End do*/

   End                                                            /* End do*/
   Call writeme " " || Substr(temp,1,Length(scanme))

End                                                               /* End do*/

/* put out 1. "source" line; 2. just brackets; 3. bracket numbers          */
/* --------------------------------------------------------------          */

If (pos(bs, linex) <> 0) Then Do          /* see to "\"chars as WINLINE .. */
   Do i = Length(linex) To 1 By -1          /* ..uses them as control char */
      If (substr(linex,i,1) = bs) Then Do
         linex = insert(bs,linex,i)           /* double the backslash char */
      End                                                        /* end if */
   End                                                           /* end do */
End                                                              /* end if */

Call writeme ":" || linex    /* ":" = position cursor in file to this line */
Call writeme " " || scanme
Call writeme " " || shadow

'CURSOR' save_row save_col            /* reposition cursor to where it was */

If (use='FILE') Then Do
   'X' tempfile
   'ALT 0 0'                                 /* claim that nothing changed */
   'CURSOR' position_cursor+1 '1'
End

/* ----------------------------------------------------------------------- */
/* Normal (successful) exit from this macro                                */
/* ----------------------------------------------------------------------- */

Exit 0                                  /* if O.K., this is the final exit */

/* ----------------------------------------------------------------------- */
/* Utility subroutine to write a line to either a file or the window.      */
/* ----------------------------------------------------------------------- */

writeme:
   Parse Arg linetowrite
   lines_written = lines_written + 1
   If (use='FILE') Then Do        /* for 1st call, cursor alredy in line 1 */
      'X' tempfile                                   /* go to results file */
      'INPUT' Substr(linetowrite,2)     /* insert new line, remove "."|":" */
      'CURSOR +1 1'                         /* down one line for next time */
      If (left(linetowrite,1) = ":") Then Do
         position_cursor = lines_written

      End
      'X' edited_file                       /* return to file being edited */
   End
   Else Do
      'WINLINE' Substr(linetowrite,2) || '\nREFRESH'
      If (rc<>0) Then Do
         Call beepit
         'MESSAGEBOX' myname': rc' rc 'from WINLINE, exiting'
         Exit rc
      End
   End                                                         /* end else */
Return

/* ----------------------------------------------------------------------- */
/* Utility subroutine to sound the beeper, usually for error situations.   */
/* ----------------------------------------------------------------------- */

beepit:
        If (sound_beep = 'NO') Then Return
        Call beep 262, 250           /* middle C for a quarter of a second */
        Return

/* End of macro SHOWNEST --------------------------------------------------*/
/* Roger Spencer                    REXX compiler team     */
